/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: ExponentialBackoffLogics
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: IBackoffLogic.h,v 1.2 2009/07/30 17:40:34 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_IBACKOFF_LOGIC_H_
#define GLITE_DATA_SRM_UTIL_CPP_IBACKOFF_LOGIC_H_

#include <boost/shared_ptr.hpp>
#include <time.h>

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace details{

/** Interface for the backoff/timeout logic when the SRM server is busy */
struct IBackoffLogic
{
	typedef boost::shared_ptr<IBackoffLogic> Ptr;

	virtual ~IBackoffLogic() { /* EMPTY */ }

	virtual bool timeout() const = 0;
	virtual void reset(const time_t* t = NULL) = 0;
	virtual void sleep() = 0;
	virtual void setSleepTime(unsigned int t) = 0;
	virtual void setMaxSleepTime(unsigned int t) = 0;
	virtual void setTimeout(unsigned int t) = 0;
    virtual void forceTimeout() = 0;
	virtual IBackoffLogic* clone() const = 0;
};

} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite
} // End namespace details

#endif //GLITE_DATA_SRM_UTIL_CPP_IBACKOFF_LOGIC_H_
