/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: Request Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: Request.h,v 1.4 2009/06/29 14:09:24 szamsu Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_DETAILS_REQUEST_H_
#define GLITE_DATA_SRM_UTIL_CPP_DETAILS_REQUEST_H_

#include "glite/data/srm/util/Context.h"
#include "glite/data/srm/util/types.h"
#include "glite/data/srm/util/SrmExceptions.h"
#include "glite/data/srm/util/details/IBackoffLogic.h"
#include <boost/utility.hpp>

namespace glite  {
namespace data   {
namespace srm    {
namespace util   {
namespace details{

/**
 * SRM Request Abstract Class
 */
class Request: boost::noncopyable {
public:

    /**
     * The SRM Context
     */
    const Context& ctx() const;

    /**
     * The SRM Context
     */
    Context& ctx();

	/** Getter of the backoff/retry/timeout logic */
	IBackoffLogic::Ptr getBackoffLogic() const;

	/** Setter of the backoff/retry/timeout logic */
	void setBackoffLogic(IBackoffLogic::Ptr logic);

protected:

    /**
     * Constructor
     */
    explicit Request(Context::Ptr ctx);

private:
	/**
     * Default Constructor. Not Implemented
     */
    Request();

private:
    /**
     * The SRM Context
     */
    Context::Ptr m_ctx;
	IBackoffLogic::Ptr m_backoffLogic;
};

} // End namespace details
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_DETAILS_REQUEST_H_
