/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: ExponentialBackoffLogics
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: BackoffLogic.h,v 1.1 2009/07/30 17:40:34 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_BACKOFF_LOGIC_H_
#define GLITE_DATA_SRM_UTIL_CPP_BACKOFF_LOGIC_H_

#include "glite/data/srm/util/details/IBackoffLogic.h"

using namespace glite::data::srm::util::details;

/** The logic */
class BackoffLogic : public IBackoffLogic
{
public:
	BackoffLogic(double maxSleepTime);
	virtual ~BackoffLogic();

	/** Set the sleep time for the next sleep call. This call will reset the
	 * exponential wait logic. This time is also added to the global wait time,
	 * so the next sleep max cause timeout. */
	virtual void setMaxSleepTime(unsigned int t);

	static const unsigned int NoMaxSleepTime() { return 0; }

protected:
	unsigned int _maxSleepTime;
};

#endif //GLITE_DATA_SRM_UTIL_CPP_BACKOFF_LOGIC_H_
