/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP : ErrorPatterns class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ErrorPatterns.h,v 1.2 2007/11/05 10:36:52 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_SRM_UTIL_CPP_ERROR_PATTERNS_H_
#define GLITE_DATA_SRM_UTIL_CPP_ERROR_PATTERNS_H_

#include "glite/data/agents/extra/XmlPatternsList.h"
#include "glite/data/srm/util/types.h"

#include <map>
#include <string>

namespace glite {
namespace data  {
namespace srm   {
namespace util  {

/**
 * ErrorPatterns
 * Wrap the XmlPatternsList for gsoap and srm1.1 errors
 */
class ErrorPatterns{
public:
    /**
     * Singleton instance
     */
    static ErrorPatterns& instance();
    /**
     * return true if given message corresponds to a GSI authentication error 
     * pattern
     * @param msg [IN] the message to match
     * @return true if match found, false otherwise
     */
    bool isGsiAuthenticationError(const std::string& msg);
    /**
     * return true if given message corresponds to a TCP connection error
     * pattern
     * @param msg [IN] the message to match
     * @return true if match found, false otherwise
     */
    bool isTcpConnectionError(const std::string& msg);
    /**
     * Translate given message to an SrmStatus code applying internal patterns.
     * @param msg [IN] the message to match
     * @return SrmStatus code (SRM_FAILURE if no match found).
     */
    SrmStatus findSrmError(const std::string& msg);
private:
    /**
     * constructor
     */
    ErrorPatterns();
    /**
     * destructor
     */
    ~ErrorPatterns();

    // map for SRM error codes
    typedef std::map<std::string,SrmStatus::Code> SrmCategoriesMap;
    SrmCategoriesMap m_srmCategoriesMap;

    // the patterns list 
    glite::data::agents::XmlPatternsList m_patterns;
};

} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif // GLITE_DATA_SRM_UTIL_CPP_ERROR_PATTERNS_H_
