/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: Synch Request Class
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: Request.cpp,v 1.1 2009/06/21 17:11:47 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "ExponentialBackoffLogic.h"
#include "glite/data/srm/util/details/Request.h"

namespace glite  {
namespace data   {
namespace srm    {
namespace util   {
namespace details{

Request::Request(Context::Ptr ctx)
	: m_ctx(ctx),
	  m_backoffLogic(new ExponentialBackoffLogic<>)
{
	// EMPTY
}

const Context& Request::ctx() const
{
    if(0 == m_ctx.get()){
        throw NullPointerException("null srm context");
    }
    return *m_ctx;
}

Context& Request::ctx()
{
     if(0 == m_ctx.get()){
         throw NullPointerException("null srm context");
     }
     return *m_ctx;
}

IBackoffLogic::Ptr Request::getBackoffLogic() const
{
	return m_backoffLogic;
}

void Request::setBackoffLogic(IBackoffLogic::Ptr logic)
{
	m_backoffLogic = logic;
}

} // End namespace details
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite
