/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmLs Class
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: SrmAsyncOperation.h,v 1.1 2009/07/30 17:45:39 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_SRM_ASYNC_OPERATION_H_
#define GLITE_DATA_SRM_UTIL_CPP_SRM_ASYNC_OPERATION_H_

#include "SrmOperation.h"
#include "glite/data/srm/util/details/IBackoffLogic.h"
#include "glite/data/srm/util/types.h"

using namespace glite::data::srm::util::details;

namespace glite {
namespace data  {
namespace srm   {
namespace util  {

template <class REQUEST, class BASE, class CONTEXT>
class SrmAsyncOperation : public BASE,
                          public SrmOperation
{
protected:
	SrmAsyncOperation(REQUEST& self, typename CONTEXT::Ptr ctx)
		: BASE(ctx),
		  _self(self)
	{
		// EMPTY
	}

	SrmAsyncOperation(REQUEST& self, typename CONTEXT::Ptr ctx, const std::string& token)
		: BASE(ctx, token),
		  _self(self)
	{
		// EMPTY
	}

	virtual ~SrmAsyncOperation()
	{
		// EMPTY
	}

public:
	virtual bool backoff()
	{
		IBackoffLogic& logic = *(_self.getBackoffLogic());
        int eWT = _self.estimatedWaitTime();

        if (eWT > 0) {
            logic.setSleepTime(eWT);
        } else if (eWT == 0) {
            // This case means that the request timed out on the server side. See:
            // https://sdm.lbl.gov/srm-wg/doc/SRM.v2.2.html
            logic.forceTimeout();
        }

		if (!logic.timeout() && !_isFinished()) 
        {
		    logic.sleep();

			if (!logic.timeout())
				_self.update();
		}

		bool isFinished = _isFinished();

		if (logic.timeout()) {
			_self.abort();
			_self.status = SrmStatus(SrmStatus::SRM_REQUEST_TIMED_OUT, "Request timeout (internal error or too long processing), request aborted");
			isFinished = true;
		}

		return isFinished;
	}

protected:
	SrmAsyncOperation() { /* EMPTY */ }

private:
	bool _isFinished() const
	{
		return ! (_self.status.code == SrmStatus::SRM_INTERNAL_ERROR ||
			      _self.status.code == SrmStatus::SRM_REQUEST_QUEUED ||
			      _self.status.code == SrmStatus::SRM_REQUEST_INPROGRESS);
	}

	REQUEST& _self;
};

} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif // #ifndef GLITE_DATA_SRM_UTIL_CPP_SRM_ASYNC_OPERATION_H_
