/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmError Exception Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SrmError.cpp,v 1.4 2008/03/13 16:32:45 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/srm/util/SrmExceptions.h"

#include <sstream>
using namespace glite::data::srm::util;

/*
 * SrmError
 *
 * Consrtuctor
 */
SrmError::SrmError(SrmStatus::Code code, const std::string& reason): 
    RuntimeError(SrmError::getErrorMessage(code,reason)),
    m_status(code,reason){    
}

/*
 * SrmError
 *
 * Consrtuctor
 */
SrmError::SrmError(const SrmStatus& status): 
    RuntimeError(SrmError::getErrorMessage(status.code,status.message)),
    m_status(status){
}
    
/*
 * getErrorMessage
 *
 * Consruct a message from an code and a description
 */
std::string SrmError::getErrorMessage(SrmStatus::Code code, const std::string& reason) /*throw()*/ {
    const char * code_str = SrmStatus::toString(code);
    
    std::stringstream ss;
    ss << "[" << code_str << "] " << reason;
    return ss.str();    
}
