/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: GSOAP Common functions
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: gsoap-common.cpp,v 1.1 2008/05/26 09:41:52 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include <glite/data/srm/util/gsoap-common.h>
#include <arpa/inet.h>
#include <sstream>

namespace {

/**
 * get_address (IPV4)
 * @param peer [IN]
 * @return address as string
 */
std::string get_address(struct soap * soap,sockaddr_in * peer){
    char buf[INET_ADDRSTRLEN];
    const char * res = inet_ntop(AF_INET,(const void *)&peer->sin_addr,buf,INET_ADDRSTRLEN);
    if (0 == res){
        std::string ret;
        return ret;
    } else {
        std::string ret(res);
        return ret;
 
  }
}

/**
 * get_address (IPV6)
 * @param peer [IN]
 * @return address as string
 */
std::string get_address(struct soap * soap,sockaddr_storage * peer){
    if (AF_INET6 == peer->ss_family){ // IPV6
        sockaddr_in6 * sockaddr6 = (sockaddr_in6 *)peer;
        char buf[INET6_ADDRSTRLEN];
        const char * res = inet_ntop(AF_INET6,(const void *)&sockaddr6->sin6_addr,buf,INET6_ADDRSTRLEN);
        if (0 == res){
            std::string ret;
            return ret;
        } else {
            std::string ret(res);
            return ret;
        }
    } else if (AF_INET == peer->ss_family){ // IPV4
        sockaddr_in * sockaddr = (sockaddr_in *)peer;
        if(sockaddr)
          return get_address(soap,sockaddr);
        else
	{
        std::stringstream ss;
        ss << "Address family: ";
        #define SS_ADDR(a) case a: ss << #a; break
        switch (peer->ss_family){
        SS_ADDR(AF_UNIX);
        SS_ADDR(AF_UNSPEC);
        default:
            ss << "unknown";
            break;
        }
        ss << " host: " << soap->host;
        ss << " ip: " << soap->ip;
        return ss.str();
	}
    }
    else {
        std::stringstream ss;
        ss << "Address family: ";
#define SS_ADDR(a) case a: ss << #a; break        
        switch (peer->ss_family){
        SS_ADDR(AF_UNIX);
        SS_ADDR(AF_UNSPEC);
        default:
            ss << "unknown";
            break;
        }
        ss << " host: " << soap->host;
        ss << " ip: " << soap->ip;
        return ss.str();
    }
}

} // namespace

/**
 * function to detect RPC timeout errors
 */ 
bool glite::data::srm::util::is_TIMEOUT_ERROR(struct soap * soap){
    if((0 != soap) && (0 != soap->fault)){
        if(SOAP_EOF == soap->error && 0 == soap->errnum) {
            return true;
        }
    }
    return false;
}


/**
 * get_ip.
 * Call appropriate overload according to type of soap->peer, which is sockaddr_storage
 * id compiling with -DWITH_IPV6 and sockaddr_in otherwise
 * @param soap [IN] the soap struct
 * @return address as string
 */
std::string glite::data::srm::util::get_ip(struct soap * soap){
    return get_address(soap,&(soap->peer));
}

