/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: AbortFiles Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: AbortFiles.cpp,v 1.3 2008/11/20 11:44:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "AbortFiles.h"
#include "SrmVersion.h"
#include "GSoapContext.h"
#include "srm-utility.h"
#include "srm-common-abortfiles.h"

using namespace glite::data::srm::util::srm1_1;

DEFINE_REQUEST_FACTORY(AbortFiles, 1_1);

/**
 * Execute the Request
 */
glite::data::srm::util::SrmStatus AbortFiles::execute() /* throw (LogicError, RemoteError, SecurityError, SrmError)*/{

    SrmStatus result;
    if(true == token.empty()){
        throw InvalidArgumentException("empty request token");
    }
    if(true == files.empty()){
        throw InvalidArgumentException("empty files array");
    }
    
    int request_id = -1;
    try{
        request_id = boost::lexical_cast<int>(this->token);
    } catch(const boost::bad_lexical_cast&){
        throw LogicError("invalid request id: not a number");
    }
    
    GSoapContext g(this->ctx(),false);
      
    // Check that the fileIds vector is filled
    if(true == this->fileIds.empty()){
        // Get the FileIds from the request
        
        // Invoke SRM RequestStatus
        struct ::srm::srm__getRequestStatusResponse outq;
        g.ctx().beforeCall("srm__getRequestStatus");
        if(SOAP_OK != g.soap().srm__getRequestStatus(request_id, outq)){
            g.ctx().onFailure("srm__getRequestStatus",g.ip().c_str(),0);
            // Failed to invoke remote GetRequestStatus: throw the correct exception
            g.handleError("SrmGetRequestStatus");
        }
        g.ctx().onSuccess("srm__getRequestStatus",g.ip().c_str(),0);
    
        // Update Request Status
        update_request_status(*this,outq._Result);
    }
    // For each file, check the status
    bool has_success = false;
    bool has_failed  = false;
    std::vector<int>::iterator it;
    for(it = this->fileIds.begin(); it != this->fileIds.end(); ++it){
        int file_id = (*it);
        AbortFileRequest * file = 0;
        file = &(this->files[std::distance(this->fileIds.begin(),it)]);
        // Set file Status to failed
        set_file_failed<AbortFiles,AbortFileRequest>(g, request_id, file_id,*this,file);
        if(file->status.code == SrmStatus::SRM_SUCCESS){
            has_success = true;
        } else {
            has_failed = true;
        }
    }
    if(true == has_success){
        if(false == has_failed){
            // Everything went OK
            result.message.clear();
            result.code = SrmStatus::SRM_SUCCESS;
        } else {
            // Some Files failed
            result.message = "The abort of some files failed";
            result.code = SrmStatus::SRM_PARTIAL_SUCCESS;
        }
    } else {
        // All Files failed
        result.message = "AbortFiles failed";
        result.code = SrmStatus::SRM_FAILURE;
    }
    return result;
}
