/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: PrepareToPut Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PrepareToPut.h,v 1.6 2009/07/30 17:40:34 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_1_1_PREPARE_TO_PUT_H_
#define GLITE_DATA_SRM_UTIL_CPP_1_1_PREPARE_TO_PUT_H_

#include "glite/data/srm/util/PrepareToPut.h"
#include "GSoapContext.h"
#include "SrmVersion.h"
#include "SrmAsyncOperation.h"

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm1_1{

/*
 * Forward Declaration
 */
class GSoapContext;

/**
 * Srm 1.1 Implementation of the PrepareToPut Asynchronous requests
 */
class PrepareToPut : public SrmAsyncOperation<PrepareToPut, glite::data::srm::util::PrepareToPut, Context>
{
	typedef SrmAsyncOperation<PrepareToPut, glite::data::srm::util::PrepareToPut, Context> _BASETYPE;

public:
    // -------------------------------------------------------------------------
    // Declare the factory method
    // -------------------------------------------------------------------------
	DECLARE_REQUEST_FACTORY(PrepareToPut, glite::data::srm::util::PrepareToPut, SRM_VERSION_1_1, 1_1)
	DECLARE_ASYNCH_REQUEST_FACTORY(PrepareToPut, glite::data::srm::util::PrepareToPut, SRM_VERSION_1_1, 1_1)

public:
    // -------------------------------------------------------------------------
    // The Request class abstract methods
    // -------------------------------------------------------------------------

    /**
     * Execute the Request
     */
    virtual void execute() /* throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/;

    // -------------------------------------------------------------------------
    // The AsynchRequest class abstract methods
    // -------------------------------------------------------------------------

    /**
     * Update the status of the Request.
     */
    virtual void update() /*throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/;

    /**
     * Complete a Request and release the allocated resources.
     */
    virtual void done() /*throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/;

    /**
     * Abort a Request and cleanup the allocated resources.
     */
    virtual void abort() /*throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/;

protected:
    explicit PrepareToPut(Context::Ptr ctx)
		: _BASETYPE(*this, ctx)
    {
    	// EMPTY
    }

    PrepareToPut(Context::Ptr ctx, const std::string& token)
		: _BASETYPE(*this, ctx, token)
    {
    	// EMPTY
    }

private:
	/**
     * Default Constructor. Not Implemented
     */
    PrepareToPut();


public:
	// -------------------------------------------------------------------------
    // SRM Put v1.1 specific: public within this module
    // -------------------------------------------------------------------------

    using glite::data::srm::util::PrepareToPut::token;
    /**
     * Save the request token
     */
    void token(const std::string& tkn){m_token = tkn;}

    /**
     * Keep track of the File Ids
     */
    std::vector<int>  fileIds;
};

} // End namespace srm1_1
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_1_1_PREPARE_TO_PUT_H_
