/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmCopy Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: SrmCopy.cpp,v 1.7 2008/11/20 11:44:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "SrmCopy.h"
#include "SrmVersion.h"
#include "GSoapContext.h"
#include "srm-utility.h"
#include "srm-common-copy.h"
#include "srm-common.h"
#include "context-caller.h"

#include <boost/lexical_cast.hpp>

using namespace glite::data::srm::util::srm1_1;

DEFINE_REQUEST_FACTORY(SrmCopy, 1_1);
DEFINE_ASYNCH_REQUEST_FACTORY(SrmCopy, 1_1);

/**
 * Execute the Request
 */
void SrmCopy::execute() /* throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/{
    CallInfo info;
    // CheckPreconditions
    this->checkToken(false);
    if(true == files.empty()){
        throw InvalidArgumentException("empty files array");
    }
    if(false == ctx().ignoreMissingFeatures){
        // Check that advanced features (not available in srm1.1) are 
        // not required
        // TODO: Check features not supported in srm1.1
    }
    
    GSoapContext g(this->ctx(),true);
                                             
    ::srm::ArrayOfstring sourcearray;
    ::srm::ArrayOfstring destarray;
    ::srm::ArrayOfboolean donearray;

    sourcearray.__size = files.size();
    destarray.__size   = files.size();
    donearray.__size   = files.size();
    sourcearray.__ptr = (char **)malloc(sizeof(char *)* sourcearray.__size);
    destarray.__ptr = (char **)malloc(sizeof(char *)  * destarray.__size);
    donearray.__ptr = (bool *)malloc(sizeof(bool)     * donearray.__size);

    std::vector<CopyFileRequest>::iterator it;
    char ** src_ptr    = &(sourcearray.__ptr[0]);
    char ** dst_ptr    = &(destarray.__ptr[0]);
    int i = 0;
    for(it = files.begin(); it != files.end(); ++it, ++src_ptr, ++dst_ptr, ++i){
        info.add("sourceSurl",(*it).sourceSurl);
        info.add("targetSurl",(*it).targetSurl);
        (*src_ptr) = &(*((*it).sourceSurl.begin()));
        (*dst_ptr) = &(*((*it).targetSurl.begin()));
         donearray.__ptr[i] = true;
    }

    // Invoke SRM Copy
    struct ::srm::srm__copyResponse outq;
    this->ctx().beforeCall("srm__copy");
    if(SOAP_OK != g.soap().srm__copy(&sourcearray, &destarray, &donearray, outq)){
        this->ctx().onFailure("srm__copy",g.ip().c_str(),&info);
        // Failed to invoke remote Put: throw the correct exception
        g.handleError("SrmCopy");
    }
    CallOnSuccess call(this->ctx(),"srm__copy",g.ip().c_str(),&info);
    // Update Request Status
    update_request_status<SrmCopy,CopyFileRequest>(*this,outq._Result);
    info.token(m_token);
}

/*
 * update
 *
 * Update the status of the Request.
 */
void SrmCopy::update() /*throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/{
    // CheckPreconditions
    this->checkToken(true);

    // Create gsoap context
    GSoapContext g(this->ctx(),false);

    // Update Request Status
    send_update_request<SrmCopy,CopyFileRequest>(g,*this);
}

/*
 * done
 *
 * Complete a Request and release the allocated resources.
 */
void SrmCopy::done() /*throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/{
    // CheckPreconditions
    this->checkToken(true);
    
    // Create gsoap context
    GSoapContext g(this->ctx(),false);
    
    // Send done request    
    send_done_request<SrmCopy,CopyFileRequest>(g,*this);
}

/*
 * abort
 *
 * Abort a Request and cleanup the allocated resources.
 */
void SrmCopy::abort() /*throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/{
    // CheckPreconditions
    this->checkToken(true);

    // Create gsoap context
    GSoapContext g(this->ctx(),false);
        
    // Send abort request    
    send_abort_request<SrmCopy,CopyFileRequest>(g,*this);
}
