/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmPing Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: SrmPing.cpp,v 1.6 2008/11/20 11:44:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "SrmPing.h"
#include "SrmVersion.h"
#include "GSoapContext.h"

using namespace glite::data::srm::util::srm1_1;

DEFINE_REQUEST_FACTORY(SrmPing, 1_1);

/**
 * Execute the Request
 */
glite::data::srm::util::SrmStatus SrmPing::execute() /* throw (LogicError, RemoteError, SecurityError, SrmError)*/{

    SrmStatus result;
    
    GSoapContext g(this->ctx(),false);
                                                                                                                                                       
    // Invoke srm__ping
    // ---------------------------------------------------------------------
    bool ping_out;
    this->ctx().beforeCall("srm__ping");
    if(SOAP_OK != g.soap().srm__ping(ping_out)){
        this->ctx().onFailure("srm__ping",g.ip().c_str(),0);
        // Failed to invoke remote Ping: throw the correct exception
        g.handleError("SrmPing");
    }
    this->ctx().onSuccess("srm__ping",g.ip().c_str(),0);
    
    // Ping succeeded: Srm 1.1 ping doesn't return any value: hard code it
    this->versionInfo = SRM_VERSION_V_1_1;
    
    result.code = SrmStatus::SRM_SUCCESS;
    return result;
}
