/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmRmDir Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: SrmRmDir.cpp,v 1.8 2008/11/20 11:44:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "SrmRmDir.h"
#include "SrmVersion.h"
#include "GSoapContext.h"
#include "srm-utility.h"

using namespace glite::data::srm::util::srm1_1;

DEFINE_REQUEST_FACTORY(SrmRmDir, 1_1);

/**
 * Execute the Request
 */
glite::data::srm::util::SrmStatus SrmRmDir::execute() /* throw (LogicError, RemoteError, SecurityError, SrmError)*/{

    SrmStatus result;
    CallInfo info;
    // In Srm 1.1 RmDir doesn't exist: Simulate it using advisory delete.

    if(false == ctx().ignoreMissingFeatures){
        // Check that advanced features (not available in srm1.1) are 
        // not required
        // TODO: Check features not supported in srm1.1
    }
    
    GSoapContext g(this->ctx(),false);
                           
    ::srm::ArrayOfstring surlarray;
                                                                                                                                                       
    // Set the SURL array
    surlarray.__size = 1;
    surlarray.__ptr = (char**)soap_malloc(g.soap().soap,sizeof(char *));
    char ** ptr = &(surlarray.__ptr[0]);
    (*ptr) = &(*(directoryPath.begin()));
    info.add("surl",directoryPath);
    // Invoke srm__advisoryDelete
    // ---------------------------------------------------------------------
    ::srm::srm__advisoryDeleteResponse out;
    this->ctx().beforeCall("srm__advisoryDelete");
    if(SOAP_OK != g.soap().srm__advisoryDelete(&surlarray, out)){
        this->ctx().onFailure("srm__advisoryDelete",g.ip().c_str(),&info);
        // Failed to invoke remote call: throw the correct exception
        try{
            g.handleError("SrmRmDir");
        } catch(const SrmError& exc){
            // In dCache, an error is returned when the file doesn't exist
            result = exc.status();
        }
    } else {
        this->ctx().onSuccess("srm__advisoryDelete",g.ip().c_str(),&info);
    
        // Set the result status
        result.code = SrmStatus::SRM_SUCCESS;
    }
    return result;
}
