/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: srm common functions
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: srm-common-defs.h,v 1.9 2008/11/20 11:44:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_1_1_SRM_COMMON_DEFS_H_
#define GLITE_DATA_SRM_UTIL_CPP_1_1_SRM_COMMON_DEFS_H_

#include "GSoapContext.h"
#include "srm-utility.h"
#include <boost/static_assert.hpp>

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm1_1{
   
/**
 * Translate Request State
 */
template<typename R>
inline void translate_request_status(R& request, char * state, const char * errorMessage) /* throw (LogicError,SrmBadResponse)*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
}

/**
 * Return true if the FileStatus structure is related to the given file
 */
template<typename F>
inline bool match_file(const F& file, const ::srm::srm2__RequestFileStatus * file_stat) {
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
    return false;
}

/**
 * Return true if the FileMetadata structure is related to the given file
 */
template<typename F>
inline bool match_file(const F& file, const ::srm::srm2__FileMetaData * file_metadata) {
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
    return false;
}

/**
 * Specialization for FileRequest
 */
template<>
inline bool match_file(const FileRequest& file, const ::srm::srm2__FileMetaData * file_metadata) {
    bool result = false;
    if(0 != file_metadata){
        const char * req_surl = file_metadata->SURL;
        if((0 != req_surl) && (strlen(req_surl) > 0)){
            // Compare the Source SURL
            if(0 == file.surl.compare(req_surl)){
                // Match Found
                result = true;
            }
        } else {
            // No SURLs returned
        }
    }
    return result;
}

/**
 * Return true if the Request Status code is success after the execute command
 */
template<typename R>
inline bool is_request_succeeded(const R& request) {
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    return false;
}

/**
 * Return true if the File Status code is pending
 */
template<typename F>
inline bool is_file_pending(const F& file) {
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
    return false;
}

/**
 * Return true if the File Status code is success after the execute command
 */
template<typename F>
inline bool is_file_succeeded(const F& file) {
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
    return false;
}

/**
 * Return true if the File Status code is released after the done command
 */
template<typename F>
inline bool is_file_released(const F& file) {
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
    return false;
}

/**
 * Get File State From Request state
 */
template<typename R, typename F>
inline void file_from_request_status(R& request, F& file) {
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
}

/**
 * Translate File State
 */    
template<typename R, typename F>
inline int update_file_request(R& request, F& file, const ::srm::srm2__RequestFileStatus * file_stat, const ::srm::srm2__RequestStatus * srm_stat) /*throw (LogicError, SrmBadResponse)*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
    return 0;
}

/**
 * Set File Running
 */    
inline void set_file_status(GSoapContext& g, int request_id, int file_id, const char * next_status) /*throw (RemoteError, SrmError)*/{
    if(0 != next_status){
        // Invoke Service setFileStatus
        // -------------------------------------------------------------------------
        g.ctx().beforeCall("srm__setFileStatus");
        struct ::srm::srm__setFileStatusResponse out;
        if(SOAP_OK != g.soap().srm__setFileStatus(request_id, file_id, const_cast<char*>(next_status), out)) {
            g.ctx().onFailure("srm__setFileStatus",g.ip().c_str(),0);
            g.handleError("SetFileStatus");
        }
        g.ctx().onSuccess("srm__setFileStatus",g.ip().c_str(),0);
    }
}

/**
 * Set File Running
 */    
template<typename R, typename F>
inline void set_file_running(GSoapContext& g, int request_id, int file_id, R& request, F * file) /*throw (RemoteError, SrmBadResponse, SrmError)*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
}

/**
 * Set File Done
 */    
template<typename R, typename F>
inline void set_file_done(GSoapContext& g, int request_id, int file_id, R& request, F * file) /*throw (RemoteError, SrmBadResponse, SrmError)*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
}

/**
 * Set File Failed
 */    
template<typename R,typename F>
inline void set_file_failed(GSoapContext& g, int request_id, int file_id, R& request, F * file) /*throw (RemoteError, SrmBadResponse, SrmError)*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
}

} // End namespace srm1_1
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_1_1_SRM_COMMON_DEFS_H_
