/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: srm 1.1 utilities
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: srm-utility.h,v 1.6 2006/11/28 12:01:16 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_1_1_SRM_UTILITY_H_
#define GLITE_DATA_SRM_UTIL_CPP_1_1_SRM_UTILITY_H_

#include "glite/data/srm/util/types.h"
#include "glite/data/srm/cpp/srmStub.h"

// Fix ugly gsoap2.7 names
namespace srm{
typedef srm4ArrayOfstring        ArrayOfstring;
typedef srm5ArrayOflong          ArrayOflong;
typedef srm5ArrayOfboolean       ArrayOfboolean;
}

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm1_1{
    
extern const char * const SRM_1_1_STATUS_PENDING;
extern const char * const SRM_1_1_STATUS_READY;
extern const char * const SRM_1_1_STATUS_ACTIVE;
extern const char * const SRM_1_1_STATUS_RUNNING;
extern const char * const SRM_1_1_STATUS_DONE;
extern const char * const SRM_1_1_STATUS_FAILED;
    
/**
 * Extract the file message from the request message using the file_id. In case
 * no file message is available, return an empty string
 */
std::string get_file_message(int fi1e_id, const char * msg) /*throw ()*/;

/**
 * Extract the file message from the request message using the file name. In 
 * case no file message is available, return an empty string
 */
std::string get_file_message(const std::string& filename, const char * msg) /*throw ()*/;

/**
 * Extract the file path from a SURL
 */
std::string get_file_path(const std::string& surl) /*throw ()*/;

// -----------------------------------------------------------------------------
// Get helpers
// -----------------------------------------------------------------------------
    
/**
 * Update the GetFileRequest with the content of SRM response
 * @return the file id
 */
int update_getfilerequest(GetFileRequest& file, const ::srm::srm2__RequestFileStatus * file_stat, const ::srm::srm2__RequestStatus * srm_stat) /*throw (LogicError, SrmBadResponse)*/;

/**
 * Translate the status of a get request
 */
SrmStatus::Code translate_getrequest_status(const char * state, const char * errorMessage) /* throw (LogicError,SrmBadResponse)*/;

/**
 * Translate the file status of a get request
 */    
SrmStatus::Code translate_getfile_status(const char * state, const char * errorMessage) /* throw (LogicError,SrmBadResponse)*/ ;

/**
 * Return the correct file status in case the response does not contain any file
 */
SrmStatus::Code file_from_getrequest_status(SrmStatus::Code req_status) /*throw (SrmBadResponse)*/;

// -----------------------------------------------------------------------------
// Put helpers
// -----------------------------------------------------------------------------
    
/**
 * Update the PutFileRequest with the content of SRM response
 * @return the file id
 */
int update_putfilerequest(PutFileRequest& file, const ::srm::srm2__RequestFileStatus * file_stat, const ::srm::srm2__RequestStatus * srm_stat) /*throw (LogicError, SrmBadResponse)*/;

/**
 * Translate the status of a put request
 */
SrmStatus::Code translate_putrequest_status(const char * state, const char * errorMessage) /* throw (LogicError,SrmBadResponse)*/;

/**
 * Translate the file status of a put request
 */    
SrmStatus::Code translate_putfile_status(const char * state, const char * errorMessage) /* throw (LogicError,SrmBadResponse)*/ ;

/**
 * Return the correct file status in case the response does not contain any file
 */
SrmStatus::Code file_from_putrequest_status(SrmStatus::Code req_status) /*throw (SrmBadResponse)*/;

// -----------------------------------------------------------------------------
// Copy helpers
// -----------------------------------------------------------------------------
    
/**
 * Update the CopyFileRequest with the content of SRM response
 * @return the file id
 */
int update_copyfilerequest(CopyFileRequest& file, const ::srm::srm2__RequestFileStatus * file_stat, const ::srm::srm2__RequestStatus * srm_stat) /*throw (LogicError, SrmBadResponse)*/;

/**
 * Translate the status of a copy request
 */
SrmStatus::Code translate_copyrequest_status(const char * state, const char * errorMessage) /* throw (LogicError,SrmBadResponse)*/;

/**
 * Translate the file status of a copy request
 */    
SrmStatus::Code translate_copyfile_status(const char * state, const char * errorMessage) /* throw (LogicError,SrmBadResponse)*/ ;

/**
 * Return the correct file status in case the response does not contain any file
 */
SrmStatus::Code file_from_copyrequest_status(SrmStatus::Code req_status) /*throw (SrmBadResponse)*/;

// -----------------------------------------------------------------------------
// BringOnline helpers
// -----------------------------------------------------------------------------
    
/**
 * Update the GetFileRequest with the content of SRM response
 * @return the file id
 */
int update_bringonlinefilerequest(BringOnlineFileRequest& file, const ::srm::srm2__RequestFileStatus * file_stat, const ::srm::srm2__RequestStatus * srm_stat) /*throw (LogicError, SrmBadResponse)*/;

/**
 * Translate the status of a bringonline request
 */
SrmStatus::Code translate_bringonlinerequest_status(const char * state, const char * errorMessage) /* throw (LogicError,SrmBadResponse)*/;

/**
 * Translate the file status of a bringonline request
 */    
SrmStatus::Code translate_bringonlinefile_status(const char * state, const char * errorMessage) /* throw (LogicError,SrmBadResponse)*/ ;

/**
 * Return the correct file status in case the response does not contain any file
 */
SrmStatus::Code file_from_bringonlinerequest_status(SrmStatus::Code req_status) /*throw (SrmBadResponse)*/;

// -----------------------------------------------------------------------------
// Stat helpers
// -----------------------------------------------------------------------------
    
/**
 * Update the MetadataPathDetail with the content of SRM response
 */
void update_file_metadata(MetaDataPathDetail& file, const ::srm::srm2__FileMetaData * file_metadata) /*throw (LogicError, SrmBadResponse)*/;

} // End namespace srm1_1
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_1_1_SRM_VERSION_H_
