/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: AbortFiles Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>,
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: AbortFiles.cpp,v 1.4 2009/06/21 17:09:31 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "AbortFiles.h"
#include "SrmVersion.h"
#include "GSoapContext.h"
#include "srm-common-abortfiles.h"
#include "srm-common.h"
#include "BackoffLogicExecutor.h"

using namespace glite::data::srm::util::srm2_2;
using namespace ::srm2;

DEFINE_REQUEST_FACTORY(AbortFiles, 2_2);

glite::data::srm::util::SrmStatus AbortFiles::execute()
{
	return BackoffLogicExecutor::executeSync(*this);
}

/**
 * Execute the Request
 */
glite::data::srm::util::SrmStatus AbortFiles::executeOneCall() /* throw (LogicError, RemoteError, SrmError)*/{
    SrmStatus result;

    CallInfo info;

    // Precondition checks
    if(true == token.empty()){
        throw InvalidArgumentException("empty request token");
    }
    if(true == files.empty()){
        throw InvalidArgumentException("empty files array");
    }

    GSoapContext g(this->ctx(),false);

    // Set SURL array
    ::srm2__ArrayOfAnyURI arrayOfSURLs;
    std::vector<AbortFileRequest>::iterator it;
    for(it = this->files.begin(); it != this->files.end(); ++it){
        info.add("surl",(*it).surl);
        arrayOfSURLs.urlArray.push_back((*it).surl);
    }

    // Invoke srm2__srmAbortFiles
    // ---------------------------------------------------------------------
    ::srm2__srmAbortFilesRequest req;
    req.authorizationID = 0;
    req.requestToken = this->token;
    req.arrayOfSURLs = &arrayOfSURLs;
    ::srm2__srmAbortFilesResponse_ rsp;
    this->ctx().beforeCall("srm2__srmAbortFiles");
    if(SOAP_OK != g.soap().srm2__srmAbortFiles(&req, rsp)){
        this->ctx().onFailure("srm2__srmAbortFiles",g.ip().c_str(),&info);
        // Failed to invoke remote AbortFiles: throw the correct exception
        g.handleError("AbortFiles");
    }
    this->ctx().onSuccess("srm2__srmAbortFiles",g.ip().c_str(),&info);

    // Get Result Code
    result = update_request
        <AbortFiles,AbortFileRequest,::srm2__srmAbortFilesResponse,::srm2__TSURLReturnStatus>
        (*this,rsp.srmAbortFilesResponse);

    return result;
}
