/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: GSOAP Context for Srm 2.2
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: GSoapContext.h,v 1.8 2008/05/08 12:48:26 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_2_2_GSOAP_CONTEXT_H_
#define GLITE_DATA_SRM_UTIL_CPP_2_2_GSOAP_CONTEXT_H_

#include "glite/data/srm/util/Context.h"
#include "glite/data/srm/util/types.h"

#ifndef SOAP_FMAC3S
#define SOAP_FMAC3S
#endif
#ifndef SOAP_FMAC4S
#define SOAP_FMAC4S
#endif

#include "glite/data/srm2/cpp/envStub.h"
#include "glite/data/srm2/cpp/srm2srmSoapBindingProxy.h"

#include <boost/utility.hpp>

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm2_2{

/**
 * The Gsoap SRM Client object
 */
class GSoapContext: boost::noncopyable {
public:
    /**
     * Constructor
     */
    GSoapContext(Context& ctx, bool deleg_cred);

    /**
     * Destructor
     */
    ~GSoapContext();

    /**
     * Return the embedded srm gsoap client 
     */
    ::srm2::srmSoapBinding& soap(){ return m_service; }

    /**
     * Return the embedded context
     */
    Context& ctx() {return m_ctx;}

    /**
     * Raise the appropriate Exception based on the Soap Fault. In 2.2 it 
     * should be only RemoteError
     */
    void handleError(const std::string& method) /*throw (LogicError, RemoteError, SecurityError, SrmError)*/;
    
    /**
     * get the ip address
     */
    std::string ip();
    
private:
    /**
     * Default Constructor. Not Implemented
     */
    GSoapContext();

private:
    /**
     * The srm gsoap client 
     */
    ::srm2::srmSoapBinding m_service;
    /**
     * The embedded context
     */
    Context& m_ctx;
    
};

} // End namespace srm2_2
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_2_2_GSOAP_CONTEXT_H_
