/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: GetSpaceMetaData Class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: GetSpaceMetaData.cpp,v 1.4 2009/06/21 17:09:31 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "GetSpaceMetaData.h"
#include "SrmVersion.h"
#include "GSoapContext.h"
#include "srm-utility.h"
#include "srm-common.h"
#include "srm-common-getspacemetadata.h"
#include "BackoffLogicExecutor.h"

using namespace glite::data::srm::util::srm2_2;
using namespace ::srm2;

DEFINE_REQUEST_FACTORY(GetSpaceMetaData, 2_2);

glite::data::srm::util::SrmStatus GetSpaceMetaData::execute()
{
	return BackoffLogicExecutor::executeSync(*this);
}

/**
 * Execute the Request
 */
glite::data::srm::util::SrmStatus GetSpaceMetaData::executeOneCall() /* throw (LogicError, RemoteError, SecurityError, SrmError)*/{

    SrmStatus result;

    CallInfo info;

    GSoapContext g(this->ctx(),false);

    // Create Request
    ::srm2__srmGetSpaceMetaDataRequest req;
    req.authorizationID = 0;

    ::srm2__ArrayOfString arrayOfSpaceTokens;
    for (std::vector<std::string>::iterator it=tokens.begin();it != tokens.end();++it){
        info.add("token",*it);
        arrayOfSpaceTokens.stringArray.push_back(*it);
    }
    req.arrayOfSpaceTokens = &arrayOfSpaceTokens;

    // Invoke srm2__srmGetSpaceMetaData
    // ---------------------------------------------------------------------
    ::srm2__srmGetSpaceMetaDataResponse_ rsp;
    this->ctx().beforeCall("srm2__srmGetSpaceMetaData");
    if(SOAP_OK != g.soap().srm2__srmGetSpaceMetaData(&req, rsp)){
        this->ctx().onFailure("srm2__srmGetSpaceMetaData",g.ip().c_str(),&info);
        // Failed to invoke remote call: throw the correct exception
        g.handleError("srm2__srmGetSpaceMetaData");
    }
    this->ctx().onSuccess("srm2__srmGetSpaceMetaData",g.ip().c_str(),&info);

    // Check Result
    if((0 == rsp.srmGetSpaceMetaDataResponse) ||
       (0 == rsp.srmGetSpaceMetaDataResponse->returnStatus)){
        // Error: No status returned by the SRM
        throw SrmBadResponse("no ReturnStatus returned by the SRM");
    }
    // Get Status
    result.code = translate_StatusCode(rsp.srmGetSpaceMetaDataResponse->returnStatus->statusCode);
    if(0 != rsp.srmGetSpaceMetaDataResponse->returnStatus->explanation){
        result.message = *(rsp.srmGetSpaceMetaDataResponse->returnStatus->explanation);
    }

    // Get space details
    const std::vector<srm2__TMetaDataSpace * > * details_array = 0;
    if(0 != rsp.srmGetSpaceMetaDataResponse->arrayOfSpaceDetails){
        details_array = &(rsp.srmGetSpaceMetaDataResponse->arrayOfSpaceDetails->spaceDataArray);
    }

    // Get Result Code
    result = update_request
            <GetSpaceMetaData,MetaDataSpace,::srm2__srmGetSpaceMetaDataResponse,::srm2__TMetaDataSpace>
            (*this,rsp.srmGetSpaceMetaDataResponse,details_array);

    return result;

}
