/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: GetSpaceMetaData Class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>,
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: GetSpaceMetaData.h,v 1.2 2009/06/21 17:09:31 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_2_2_GET_SPACE_META_DATA_H_
#define GLITE_DATA_SRM_UTIL_CPP_2_2_GET_SPACE_META_DATA_H_

#include "glite/data/srm/util/GetSpaceMetaData.h"
#include "GSoapContext.h"
#include "SrmVersion.h"
#include "SrmOperation.h"

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm2_2{

/**
 * Srm 2.2 Implementation of the GetSpaceMetaData Synchronous request
 */
class GetSpaceMetaData : public glite::data::srm::util::GetSpaceMetaData,
					     public SrmOperation
{
public:
    // -------------------------------------------------------------------------
    // Declare the factory method
    // -------------------------------------------------------------------------
    DECLARE_REQUEST_FACTORY(GetSpaceMetaData, glite::data::srm::util::GetSpaceMetaData, SRM_VERSION_2_2, 2_2)

public:
    // -------------------------------------------------------------------------
    // The Request class abstract methods
    // -------------------------------------------------------------------------

    /**
     * Execute the Request
     */
    virtual SrmStatus execute() /* throw (LogicError, RemoteError, SecurityError, SrmBadResponse, SrmError)*/;

    /** Call SrmLs once. The backoff logic uses this in execute as many times as needed. */
	SrmStatus executeOneCall();
protected:

    /**
     * Constructor
     */
    explicit GetSpaceMetaData(Context::Ptr ctx)
		: SrmOperation(),
		  glite::data::srm::util::GetSpaceMetaData(ctx){}

private:
    /**
     * Default constructor. Not implemented
     */
    GetSpaceMetaData();

};

} // End namespace srm2_2
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_2_2_GET_SPACE_META_DATA_H_
