/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: Copy Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: SrmCopy.h,v 1.5 2009/07/30 17:40:34 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COPY_H_
#define GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COPY_H_

#include "glite/data/srm/util/SrmCopy.h"
#include "SrmVersion.h"
#include "SrmAsyncOperation.h"

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm2_2{

/**
 * Srm 2.2 Class for Copy Asynchronous requests
 */
class SrmCopy : public SrmAsyncOperation<SrmCopy, glite::data::srm::util::SrmCopy, Context>
{
	typedef SrmAsyncOperation<SrmCopy, glite::data::srm::util::SrmCopy, Context> _BASETYPE;

public:
    // -------------------------------------------------------------------------
    // Declare the factory method
    // -------------------------------------------------------------------------
	DECLARE_REQUEST_FACTORY(SrmCopy, glite::data::srm::util::SrmCopy, SRM_VERSION_2_2, 2_2)
	DECLARE_ASYNCH_REQUEST_FACTORY(SrmCopy, glite::data::srm::util::SrmCopy, SRM_VERSION_2_2, 2_2)

public:
    // -------------------------------------------------------------------------
    // The Request class abstract methods
    // -------------------------------------------------------------------------

    /**
     * Execute the Request
     */
    virtual void execute() /* throw (LogicError, RemoteError, SrmError)*/;

    // -------------------------------------------------------------------------
    // The AsynchRequest class abstract methods
    // -------------------------------------------------------------------------

    /**
     * Update the status of the Request.
     */
    virtual void update() /*throw (LogicError, RemoteError, SrmError)*/;

    /**
     * Complete a Request and release the allocated resources.
     */
    virtual void done() /*throw (LogicError, RemoteError, SrmError)*/;

    /**
     * Abort a Request and cleanup the allocated resources.
     */
    virtual void abort() /*throw (LogicError, RemoteError, SrmError)*/;

    /** Call SrmCopy once. The backoff logic uses this in execute as many times as needed. */
    void executeOneCall();

protected:

    /**
     * name of the TExtraInfo key to use to serialize source space token
     */
    static const std::string SourceSpaceTokenKeyName;

    /**
     * name of the TExtraInfo key to use to serialize copy pin lifetime
     */
    static const std::string CopyPinLifetimeKeyName;

    explicit SrmCopy(Context::Ptr ctx)
		: _BASETYPE(*this, ctx)
    {
    	// EMPTY
    }

    SrmCopy(Context::Ptr ctx, const std::string& token)
		: _BASETYPE(*this, ctx, token)
    {
    	// EMPTY
    }
private:
	/**
     * Default Constructor. Not Implemented
     */
    SrmCopy();

public:
	// -------------------------------------------------------------------------
    // SRM Copy v2.2 specific: public within this module
    // -------------------------------------------------------------------------

    using glite::data::srm::util::SrmCopy::token;
    /**
     * Save the request token
     */
    void token(const std::string& tkn){m_token = tkn;}
};

} // End namespace srm2_2
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COPY_H_
