/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmLs Class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: SrmLs.h,v 1.4 2009/07/30 17:40:34 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_LS_H_
#define GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_LS_H_

#include "glite/data/srm/util/SrmLs.h"
#include "SrmVersion.h"
#include "SrmOperation.h"
#include "SrmAsyncOperation.h"

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm2_2{

class SrmLs :public SrmAsyncOperation<SrmLs, glite::data::srm::util::SrmLs, Context>
{
	typedef SrmAsyncOperation<SrmLs, glite::data::srm::util::SrmLs, Context> _BASETYPE;

public:
    // -------------------------------------------------------------------------
    // Declare the factory method
    // -------------------------------------------------------------------------
	DECLARE_REQUEST_FACTORY(SrmLs, glite::data::srm::util::SrmLs, SRM_VERSION_2_2, 2_2)
    DECLARE_ASYNCH_REQUEST_FACTORY(SrmLs, glite::data::srm::util::SrmLs, SRM_VERSION_2_2, 2_2)

public:
    // -------------------------------------------------------------------------
    // The AsynchRequest class abstract methods
    // -------------------------------------------------------------------------
	virtual void execute();

    /**
     * Update the status of the Request.
     */
    virtual void update() /*throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/;

    /**
     * Complete a Request and release the allocated resources.
     */
    virtual void done() /*throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/;

    /**
     * Abort a Request and cleanup the allocated resources.
     */
    virtual void abort() /*throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/;

    /** Call SrmLs once. The backoff logic uses this in execute as many times as needed. */
    void executeOneCall();

protected:
    explicit SrmLs(Context::Ptr ctx)
		: _BASETYPE(*this, ctx)
    {
    	// EMPTY
    }

    SrmLs(Context::Ptr ctx, const std::string& token)
		: _BASETYPE(*this, ctx, token)
    {
    	// EMPTY
    }

private:
	/**
     * Default Constructor. Not Implemented
     */
    SrmLs();

public:
    // -------------------------------------------------------------------------
    // SRM v2.2 specific: public within this module
    // -------------------------------------------------------------------------

    using glite::data::srm::util::SrmLs::token;
    /**
     * Save the request token
     */
    void token(const std::string& tkn){m_token = tkn;}
};

} // End namespace srm2_2
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_LS_H_
