/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmPing Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: SrmPing.cpp,v 1.9 2009/06/21 17:09:31 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "SrmPing.h"
#include "SrmVersion.h"
#include "GSoapContext.h"
#include "BackoffLogicExecutor.h"

using namespace glite::data::srm::util::srm2_2;
using namespace ::srm2;

DEFINE_REQUEST_FACTORY(SrmPing, 2_2);

glite::data::srm::util::SrmStatus SrmPing::execute()
{
	return BackoffLogicExecutor::executeSync(*this);
}

/**
 * Execute the Request
 */
glite::data::srm::util::SrmStatus SrmPing::executeOneCall() /* throw (LogicError, RemoteError, SrmError)*/{
    SrmStatus result;

    GSoapContext g(this->ctx(),false);

    // Invoke srm2__srmPing
    // ---------------------------------------------------------------------
    ::srm2__srmPingRequest  ping_req;
    ping_req.authorizationID = 0;
    ::srm2__srmPingResponse_ ping_rsp;
    this->ctx().beforeCall("srm2__srmPing");
    if(SOAP_OK != g.soap().srm2__srmPing(&ping_req, ping_rsp)){
        this->ctx().onFailure("srm2__srmPing",g.ip().c_str(),0);
        // Failed to invoke remote Ping: throw the correct exception
        g.handleError("SrmPing");
    }
    this->ctx().onSuccess("srm2__srmPing",g.ip().c_str(),0);

    if((0 == ping_rsp.srmPingResponse) ||
       (ping_rsp.srmPingResponse->versionInfo.empty())){
        // Ping succeeded but no version returned: use the default
        this->versionInfo = SRM_VERSION_2_2;
    } else {
        this->versionInfo = ping_rsp.srmPingResponse->versionInfo;
    }
    if (0 != ping_rsp.srmPingResponse && 0 != ping_rsp.srmPingResponse->otherInfo){
        std::vector<srm2__TExtraInfo * >::const_iterator it;
        const std::vector<srm2__TExtraInfo * >& infoarray = ping_rsp.srmPingResponse->otherInfo->extraInfoArray;
        for (it = infoarray.begin();it != infoarray.end();++it){
            srm2__TExtraInfo * info = (*it);
            if (0 != info){
                std::string value;
                if (0 != info->value){
                    value = *(info->value);
                }
                this->otherInfo[info->key] = value;
            }
        }
    }
    result.code = SrmStatus::SRM_SUCCESS;
    return result;
}
