/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmRm Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: SrmRm.cpp,v 1.10 2009/06/21 17:09:31 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "SrmRm.h"
#include "GSoapContext.h"
#include "srm-common-rm.h"
#include "srm-common.h"
#include "BackoffLogicExecutor.h"

using namespace glite::data::srm::util::srm2_2;
using namespace ::srm2;

DEFINE_REQUEST_FACTORY(SrmRm, 2_2);

glite::data::srm::util::SrmStatus SrmRm::execute()
{
	return BackoffLogicExecutor::executeSync(*this);
}

/**
 * Execute the Request
 */
glite::data::srm::util::SrmStatus SrmRm::executeOneCall() /* throw (LogicError, RemoteError, SrmError)*/{
    CallInfo info;
    SrmStatus result;

    GSoapContext g(this->ctx(),false);

    // Set SURL array
    ::srm2__ArrayOfAnyURI arrayOfSURLs;
    std::vector<RmFileRequest>::iterator it;
    for(it = this->files.begin(); it != this->files.end(); ++it){
        info.add("surl",(*it).surl);
        arrayOfSURLs.urlArray.push_back((*it).surl);
    }

    // Invoke srm2__srmRm
    // ---------------------------------------------------------------------
    ::srm2__srmRmRequest req;
    req.authorizationID = 0;
    req.storageSystemInfo = 0;
    req.arrayOfSURLs = &arrayOfSURLs;
    ::srm2__srmRmResponse_ rsp;
    this->ctx().beforeCall("srm2__srmRm");
    if(SOAP_OK != g.soap().srm2__srmRm(&req, rsp)){
        this->ctx().onFailure("srm2__srmRm",g.ip().c_str(),&info);
        // Failed to invoke remote SrmRm: throw the correct exception
        g.handleError("SrmRm");
    }
    this->ctx().onSuccess("srm2__srmRm",g.ip().c_str(),&info);

    // Get Result Code
    result = update_request
        <SrmRm,RmFileRequest,::srm2__srmRmResponse,::srm2__TSURLReturnStatus>
        (*this,rsp.srmRmResponse);

    return result;
}
