/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmRmDir Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: SrmRmDir.cpp,v 1.7 2009/06/21 17:09:31 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "SrmRmDir.h"
#include "SrmVersion.h"
#include "GSoapContext.h"
#include "srm-utility.h"
#include "BackoffLogicExecutor.h"

#include <sstream>

using namespace glite::data::srm::util::srm2_2;
using namespace ::srm2;

DEFINE_REQUEST_FACTORY(SrmRmDir, 2_2);

glite::data::srm::util::SrmStatus SrmRmDir::execute()
{
	return BackoffLogicExecutor::executeSync(*this);
}

/**
 * Execute the Request
 */
glite::data::srm::util::SrmStatus SrmRmDir::executeOneCall() /* throw (LogicError, RemoteError, SrmError)*/{

    SrmStatus result;

    CallInfo info;
    info.add ("surl",this->directoryPath);

    GSoapContext g(this->ctx(),false);

    // Invoke srm2__srmRmdir
    // ---------------------------------------------------------------------
    ::srm2__srmRmdirRequest req;
    req.authorizationID = 0;
    req.storageSystemInfo = 0;
    req.SURL = this->directoryPath;
    req.recursive = &(this->recursive);
    ::srm2__srmRmdirResponse_ rsp;
    this->ctx().beforeCall("srm2__srmRmdir");
    if(SOAP_OK != g.soap().srm2__srmRmdir(&req, rsp)){
        this->ctx().onFailure("srm2__srmRmdir",g.ip().c_str(),&info);
        // Failed to invoke remote method: throw the correct exception
        g.handleError("srm2__srmRmdir");
    }
    this->ctx().onSuccess("srm2__srmRmdir",g.ip().c_str(),&info);

    // Check Result
    if((0 == rsp.srmRmdirResponse) ||
       (0 == rsp.srmRmdirResponse->returnStatus)){
        // Error: No status returned by the SRM
        throw SrmBadResponse("no ReturnStatus returned by the SRM");
    }
    // Get Status
    result.code = translate_StatusCode(rsp.srmRmdirResponse->returnStatus->statusCode);
    if(0 != rsp.srmRmdirResponse->returnStatus->explanation){
        result.message = *(rsp.srmRmdirResponse->returnStatus->explanation);
    }
    // Verify Postconditions
    switch(result.code){
        case SrmStatus::SRM_SUCCESS:
        case SrmStatus::SRM_FAILURE:
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_INVALID_PATH:
        case SrmStatus::SRM_NON_EMPTY_DIRECTORY:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_NOT_SUPPORTED:
            // OK
        break;
        default:
            // Invalid result returned by the SRM: set it to generic
            // failure
            std::stringstream reason;
            reason << "Invalid Request Status returned by the SRM ["
                   << result.code << "]. "
                   << "Consider it as generic failure";
            if(false == result.message.empty()){
                reason << ". Original error message was: "
                       << result.message;
            }
            result.code = SrmStatus::SRM_FAILURE;
            result.message = reason.str();
    }

    return result;
}
