/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmStat Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: SrmStat.cpp,v 1.11 2009/06/21 17:09:31 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "SrmStat.h"
#include "SrmLs.h"
#include "SrmVersion.h"
#include "srm-utility.h"
#include "srm-common.h"
#include "BackoffLogicExecutor.h"
#include <algorithm>

using namespace glite::data::srm::util::srm2_2;
using namespace ::srm2;

DEFINE_REQUEST_FACTORY(SrmStat, 2_2);

glite::data::srm::util::SrmStatus SrmStat::execute()
{
	Context::Ptr context(new OwnContext(this->ctx()));
    boost::shared_ptr<glite::data::srm::util::srm2_2::SrmLs> req(
    		dynamic_cast<glite::data::srm::util::srm2_2::SrmLs*>(SrmLs::create(context)));

    req->setBackoffLogic(this->getBackoffLogic());

    for(std::vector<MetaDataPathDetail>::iterator it = this->files.begin();
		it != this->files.end();
		++it)
    {
        req->surls.push_back(it->surl);
    }

    req->fullDetailedList  = &(this->fullDetailedList);
    req->allLevelRecursive = 0;
    req->numOfLevels       = 0;
    req->fileStorageType   = this->fileStorageType;
    req->execute();
    glite::data::srm::util::SrmStatus result = BackoffLogicExecutor::updateAsyncUntilQueued(*req);
    this->files.clear();
	// TODO MetaDataPathDetail must have proper copy constructor!!!
	struct MetaDataPathDetail_Transform
	{
		MetaDataPathDetail& operator () (MetaDataPathDetail2& x) const
		{
			return (MetaDataPathDetail&)(x);
		}
	};

	MetaDataPathDetail_Transform op;

	for (std::vector<MetaDataPathDetail2>::iterator i = req->details.begin();
		 i != req->details.end();
		 ++i)
	{
		this->files.push_back(op(*i));
	}

    return result;
}
