/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: srm common functions
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: srm-common-defs.h,v 1.4 2008/03/13 16:32:45 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_DEFS_H_
#define GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_DEFS_H_

#include "GSoapContext.h"
#include <sstream>
#include <boost/static_assert.hpp>

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm2_2{

/**
 * Check Preconditions
 */
template<typename R, typename S>
inline void preconditions(const R& request, const S * srm_stat)  /*throw (LogicError, SrmBadResponse)*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    BOOST_STATIC_ASSERT(sizeof(S) == 0);
}

/**
 * Check Postconditions and eventually modify the request to make it consistent
 */
template<typename R, typename S>
inline void postconditions(R& request, SrmStatus& request_status, const S * srm_stat)  /*throw (LogicError, SrmBadResponse)*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    BOOST_STATIC_ASSERT(sizeof(S) == 0);
}


/**
 * Get File State From Request state
 */
template<typename R, typename F, typename S>
inline void file_from_request_status(const R& request, const SrmStatus& status, F& file, const S * srm_stat) /*throw ()*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
    BOOST_STATIC_ASSERT(sizeof(S) == 0);
}

/**
 * Return true if the File structure is related to the given file
 */
template<typename F,typename A>
inline bool match_file(const F& file, const A * file_stat) /*throw ()*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
    BOOST_STATIC_ASSERT(sizeof(A) == 0);
    return false;
}

/**
 * Update Request Properties
 */    
template<typename R, typename S>
inline void update_request_properties(R& request,const SrmStatus& status, const S * srm_result) /*throw (LogicError, SrmBadResponse)*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    BOOST_STATIC_ASSERT(sizeof(S) == 0);
}

/**
 * Update File Properties
 */    
template<typename R, typename F, typename A, typename S>
inline void update_file_properties(const R& request_status, const SrmStatus& status, F& file, const A * file_stat, const S * srm_stat) /*throw (LogicError, SrmBadResponse)*/{
    // This function should not be used
    BOOST_STATIC_ASSERT(sizeof(R) == 0);
    BOOST_STATIC_ASSERT(sizeof(F) == 0);
    BOOST_STATIC_ASSERT(sizeof(A) == 0);
    BOOST_STATIC_ASSERT(sizeof(S) == 0);
}

/**
 * Check That the token is the same
 */
template<typename R, typename S>
inline void check_token(const R& request, const S * srm_result)  /*throw (LogicError, SrmBadResponse)*/{
    if((0 != srm_result->requestToken) && (false == request.token().empty())){
        // Check that the request is the same
        if(request.token() != *(srm_result->requestToken)){
            // Error: the status of a different request has been returned
            std::stringstream reason;
            reason << "Invalid SRM Response: Received message for token ["
                   << *(srm_result->requestToken) << "] while expecting ["
                   << request.token() << "]";
            throw SrmBadResponse(reason.str());
        }
    }
}

} // End namespace srm2_2
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_DEFS_H_
