/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 * *  GLite Data SRM Util CPP: srm common functions (stat specialization)
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: srm-common-ls.h,v 1.1 2008/03/18 17:07:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_LS_H_
#define GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_LS_H_

#include "SrmLs.h"
#include "srm-utility.h"
#include "srm-common-defs.h"
#include "glite/data/agents/surl-utility.h"

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm2_2{

/**
 * get the output details array from the response. 
 * Introduced to allow handling GetSpaceMetaData (which does not have an array
 * named "files")
 */
template<>
std::vector<MetaDataPathDetail2>& get_array(SrmLs& request){
    return request.details;
}

/**
 * Update the state of a Request instance
 */
template<>
SrmStatus update_request<
    SrmLs,                                      // R
    MetaDataPathDetail2,                         // F
    ::srm2::srm2__srmStatusOfLsRequestResponse, // S
    ::srm2::srm2__TMetaDataPathDetail>(         // A
        SrmLs&                                              request, 
        const ::srm2::srm2__srmStatusOfLsRequestResponse *  srm_result) /*throw (LogicError, RemoteError, SrmBadResponse, SrmError)*/{

    const std::vector< ::srm2::srm2__TMetaDataPathDetail* > * file_array = 0;
    if ((0 != srm_result) &&
        (0 != srm_result->details)){
        file_array = &(srm_result->details->pathDetailArray);
    }
    return update_request<
        SrmLs,MetaDataPathDetail2,::srm2::srm2__srmStatusOfLsRequestResponse,::srm2::srm2__TMetaDataPathDetail>(
            request,srm_result,file_array);
}

/**
 * Check Preconditions
 */
template<>
inline void preconditions(
    const SrmLs&                         request, 
    const ::srm2::srm2__srmLsResponse *  srm_ls)  /*throw (LogicError, SrmBadResponse)*/{
    // Nothing To Check
}
    
/**
 * Check Preconditions
 */
template<>
inline void preconditions(
    const SrmLs&                                        request, 
    const ::srm2::srm2__srmStatusOfLsRequestResponse *  srm_ls)  /*throw (LogicError, SrmBadResponse)*/{
    // Nothing To Check
}

/**
 * Check Preconditions
 */
template<>
inline void preconditions(
    const SrmLs&                                    request, 
    const ::srm2::srm2__srmAbortRequestResponse *   srm_ls)  /*throw (LogicError, SrmBadResponse)*/{
    // Nothing To Check
}


/**
 * Check Postconditions
 */
template<>
inline void postconditions(
    SrmLs&                              request, 
    SrmStatus&                          request_status, 
    const ::srm2::srm2__srmLsResponse * srm_ls)  /*throw (LogicError, SrmBadResponse)*/{
    // Post-condition check
    switch(request_status.code){
        case SrmStatus::SRM_SUCCESS:
        case SrmStatus::SRM_PARTIAL_SUCCESS:
        case SrmStatus::SRM_REQUEST_QUEUED:
        case SrmStatus::SRM_REQUEST_INPROGRESS:
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_TOO_MANY_RESULTS:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_NOT_SUPPORTED:
        case SrmStatus::SRM_FAILURE:
            // OK
            break;
        default:
            // Invalid result returned by the SRM: set it to generic 
            // failure
            failure_for_invalid_result(request_status);
            break;
    }
}

/**
 * Check Postconditions
 */
template<>
inline void postconditions(
    SrmLs&                                              request, 
    SrmStatus&                                          request_status, 
    const ::srm2::srm2__srmStatusOfLsRequestResponse *  srm_ls)  /*throw (LogicError, SrmBadResponse)*/{
    // Post-condition check
    switch(request_status.code){
        case SrmStatus::SRM_SUCCESS:
        case SrmStatus::SRM_PARTIAL_SUCCESS:
        case SrmStatus::SRM_REQUEST_QUEUED:
        case SrmStatus::SRM_REQUEST_INPROGRESS:
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_TOO_MANY_RESULTS:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_NOT_SUPPORTED:
        case SrmStatus::SRM_FAILURE:
            // OK
            break;
        default:
            // Invalid result returned by the SRM: set it to generic 
            // failure
            failure_for_invalid_result(request_status);
            break;
    }
}

/**
 * Check Postconditions - AbortRequest
 */
template<>
inline void postconditions(
    SrmLs&                                          request, 
    SrmStatus&                                      request_status,
    const ::srm2::srm2__srmAbortRequestResponse *   srm_result)  /*throw (LogicError, SrmBadResponse)*/{
    // Post-condition check
    check_abort_postconditions(request_status);
}


inline void file_from_request_status_srmls(const SrmStatus& request_status,MetaDataPathDetail2& file){
    // Check the result and get the default file status
    switch(request_status.code){
        case SrmStatus::SRM_SUCCESS:
        case SrmStatus::SRM_FAILURE:
            file.status.code = request_status.code;
        break;
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_TOO_MANY_RESULTS:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_NOT_SUPPORTED:
        case SrmStatus::SRM_PARTIAL_SUCCESS:
            // Leave the file status undef
            file.status.code = SrmStatus::SRM_UNDEF;
        break;
        default:
            // This should never happen
        break;
    }
}

/**
 * Get File State From Request state
 */
template<>
inline void file_from_request_status(
    const SrmLs&                        request, 
    const SrmStatus&                    request_status, 
    MetaDataPathDetail2&                 file, 
    const ::srm2::srm2__srmLsResponse * srm_ls) {
    file_from_request_status_srmls(request_status,file);
}

/**
 * Get File State From Request state
 */
template<>
inline void file_from_request_status(
    const SrmLs&                                        request, 
    const SrmStatus&                                    request_status, 
    MetaDataPathDetail2&                                 file, 
    const ::srm2::srm2__srmStatusOfLsRequestResponse *  srm_ls) {
    file_from_request_status_srmls(request_status,file);
}

/**
 * Get File State From Request state - AbortRequest
 */
template<>
inline void file_from_request_status(
    const SrmLs&                                  request, 
    const SrmStatus&                              request_status, 
    MetaDataPathDetail2&                           file, 
    const ::srm2::srm2__srmAbortRequestResponse * srm_result) {
    // Check the result and get the default file status
    switch(request_status.code){
        case SrmStatus::SRM_SUCCESS:
            file.status.code = SrmStatus::SRM_ABORTED;
        break;
        case SrmStatus::SRM_ABORTED:
            // This shoudl not be returned by the SRM, but it's still valid
        case SrmStatus::SRM_FAILURE:
            file.status.code = request_status.code;
        break;
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_NOT_SUPPORTED:
            // Leave the file status undef
            file.status.code = SrmStatus::SRM_UNDEF;
        break;
        default:
            // This should never happen
        break;
    }
}


/**
 * Return true if the File structure is related to the given file
 */
template<>
inline bool match_file(
    const MetaDataPathDetail2&                   file, 
    const ::srm2::srm2__TMetaDataPathDetail *   file_stat) {
    bool result = false;
    if(file_stat != 0){
        if(file_stat->path == file.surl){
            result = true;
        } else {
            // Check if the returned value is just a path
            try{
                std::string file_path = glite::data::agents::path_from_surl(file.surl);
                if(file_stat->path == file_path){
                    result = true;
                }
            } catch(...){}
        }
    }
    return result;
}

/**
 * Update Request Properties
 */    
template<>
inline void update_request_properties(
    SrmLs&                              request,
    const SrmStatus&                    status, 
    const ::srm2::srm2__srmLsResponse * srm_result) /*throw (LogicError, SrmBadResponse)*/{
    // Nothing to do
}

/**
 * Update Request Properties
 */    
template<>
inline void update_request_properties(
    SrmLs&                                              request,
    const SrmStatus&                                    status, 
    const ::srm2::srm2__srmStatusOfLsRequestResponse *  srm_result) /*throw (LogicError, SrmBadResponse)*/{
    // Nothing to do
}

/**
 * Update Request Properties
 */    
template<>
inline void update_request_properties(
    SrmLs&                                          request,
    const SrmStatus&                                status, 
    const ::srm2::srm2__srmAbortRequestResponse *   srm_result) /*throw (LogicError, SrmBadResponse)*/{
    // Nothing to do
}

/**
 * Update file properties
 * @param request 
 * @param request_status 
 * @param file 
 * @param file_stat 
 */
inline void update_file_properties(
    const SrmLs&                                request, 
    const SrmStatus&                            request_status, 
    MetaDataPathDetail2&                         file, 
    const ::srm2::srm2__TMetaDataPathDetail *   file_stat){
        
    if(0 == file_stat){
        // This should never happen: file_stat has been checked before
    } else {
        // Check if the file SURL is set otherwise set it
        if(true == file.surl.empty()){
            file.surl = file_stat->path;
        }
        // Set the file Status
        if(0 != file_stat->status){
            file.status.code = translate_StatusCode(file_stat->status->statusCode);
            if(0 != file_stat->status->explanation){
                file.status.message = *(file_stat->status->explanation);
            }
            // Check that the status code  is valid
            switch(file.status.code){
                case SrmStatus::SRM_SUCCESS:
                case SrmStatus::SRM_REQUEST_INPROGRESS:
                case SrmStatus::SRM_REQUEST_QUEUED:
                case SrmStatus::SRM_INVALID_PATH:
                case SrmStatus::SRM_AUTHORIZATION_FAILURE:
                case SrmStatus::SRM_FILE_BUSY:
                case SrmStatus::SRM_FILE_LIFETIME_EXPIRED:
                case SrmStatus::SRM_FILE_IN_CACHE:
                case SrmStatus::SRM_FAILURE:
                    // Ok
                break;
                default:
                    // Invalid result returned by the SRM: set it to generic 
                    // failure
                    failure_for_invalid_result(file.status);
                break;
            }
        } else {
            // The status is not mandatory
            file.status.code = SrmStatus::SRM_UNDEF;
        }
        // Set File Properties
        if(0 != file_stat->size){
            file.size = *(file_stat->size);
        }
        if(0 != file_stat->createdAtTime){
            file.createdAtTime = *(file_stat->createdAtTime);
        }
        if(0 != file_stat->lastModificationTime){
            file.lastModificationTime = *(file_stat->lastModificationTime);
        }
        if(0 != file_stat->lastModificationTime){
            file.lastModificationTime = *(file_stat->lastModificationTime);
        }
        if(0 != file_stat->fileStorageType){
            file.fileStorageType = translate_FileStorageType(*(file_stat->fileStorageType));
        }
        if(0 != file_stat->retentionPolicyInfo){
            // RetentionPolicy is mandatory
            file.retentionPolicyInfo.retentionPolicy = translate_RetentionPolicy(file_stat->retentionPolicyInfo->retentionPolicy);
            if(0 != file_stat->retentionPolicyInfo->accessLatency){
                file.retentionPolicyInfo.accessLatency = translate_AccessLatency(*(file_stat->retentionPolicyInfo->accessLatency));
            }
        }
        if(0 != file_stat->fileLocality){
            file.fileLocality = translate_FileLocality(*(file_stat->fileLocality));
        }
        if(0 != file_stat->arrayOfSpaceTokens){
            // Swap since I don't need the data anymore
            file.spaceTokens.swap(file_stat->arrayOfSpaceTokens->stringArray);
        }
        if(0 != file_stat->type){
            file.fileType = translate_FileType(*(file_stat->type));
        }
        if(0 != file_stat->lifetimeAssigned){
            file.lifetimeAssigned = *(file_stat->lifetimeAssigned);
        }
        if(0 != file_stat->lifetimeLeft){
            file.lifetimeLeft = *(file_stat->lifetimeLeft);
        }
        if(0 != file_stat->ownerPermission){
            file.ownerPermission.userID = file_stat->ownerPermission->userID;
            file.ownerPermission.mode = translate_PermissionMode(file_stat->ownerPermission->mode);
        }
        if(0 != file_stat->groupPermission){
            file.groupPermission.groupID = file_stat->groupPermission->groupID;
            file.groupPermission.mode = translate_PermissionMode(file_stat->groupPermission->mode);
        }
        if(0 != file_stat->otherPermission){
            file.otherPermission = translate_PermissionMode(*(file_stat->otherPermission));
        }
        if(0 != file_stat->checkSumType){
            file.checkSumType = *(file_stat->checkSumType);
        }
        if(0 != file_stat->checkSumValue){
            file.checkSumValue = *(file_stat->checkSumValue);
        }
        if (0 != file_stat->arrayOfSubPaths){
            std::vector< ::srm2::srm2__TMetaDataPathDetail * >::const_iterator it;
            const std::vector< ::srm2::srm2__TMetaDataPathDetail * >& subpaths = file_stat->arrayOfSubPaths->pathDetailArray;
            for (it = subpaths.begin();it != subpaths.end();++it){
                MetaDataPathDetail2 subPath;
                update_file_properties(request,request_status,subPath,*it);
                file.arrayOfSubPaths.push_back(subPath);
            }
        }
    }
    
}

/**
 * Update File Properties
 */    
template<>
inline void update_file_properties(
    const SrmLs&                                request, 
    const SrmStatus&                            request_status, 
    MetaDataPathDetail2&                         file, 
    const ::srm2::srm2__TMetaDataPathDetail *   file_stat, 
    const ::srm2::srm2__srmLsResponse *         srm_ls) /*throw (LogicError, SrmBadResponse)*/{
    update_file_properties(request, request_status, file, file_stat);
}

/**
 * Update File Properties
 */    
template<>
inline void update_file_properties(
    const SrmLs&                                        request, 
    const SrmStatus&                                    request_status, 
    MetaDataPathDetail2&                                 file, 
    const ::srm2::srm2__TMetaDataPathDetail *           file_stat, 
    const ::srm2::srm2__srmStatusOfLsRequestResponse *  srm_ls) /*throw (LogicError, SrmBadResponse)*/{
    update_file_properties(request, request_status, file, file_stat);
}


} // End namespace srm2_2
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_STAT_H_
