/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: srm common functions (put specialization)
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: srm-common-put.h,v 1.12 2008/10/22 11:58:10 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_PUT_H_
#define GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_PUT_H_

#include "PrepareToPut.h"
#include "srm-utility.h"
#include "srm-common-defs.h"

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm2_2{

/**
 * Check Preconditions - PrepareToPut
 */
template<>
inline void preconditions(const PrepareToPut& request, const ::srm2::srm2__srmPrepareToPutResponse * srm_result)  /*throw (LogicError, SrmBadResponse)*/{
    check_token<PrepareToPut,::srm2::srm2__srmPrepareToPutResponse>(request,srm_result);
}
/**
 * Check Preconditions - StatusOfPutRequest
 */
template<>
inline void preconditions(const PrepareToPut& request, const ::srm2::srm2__srmStatusOfPutRequestResponse * srm_result)  /*throw (LogicError, SrmBadResponse)*/{
    // Nothing To Do
}
/**
 * Check Preconditions - PutDone
 */
template<>
inline void preconditions(const PrepareToPut& request, const ::srm2::srm2__srmPutDoneResponse * srm_result)  /*throw (LogicError, SrmBadResponse)*/{
    // Nothing To Do
}
/**
 * Check Preconditions - AbortRequest
 */
template<>
inline void preconditions(const PrepareToPut& request, const ::srm2::srm2__srmAbortRequestResponse * srm_result)  /*throw (LogicError, SrmBadResponse)*/{
    // Nothing To Do
}

/*
 * Check PrepareToPut Postconditions - Used in PrepareToPut and StatusOfPutRequest
 */
inline void put_postconditions(PrepareToPut& request, SrmStatus& request_status)  /*throw (LogicError, SrmBadResponse)*/{
    // Post-condition check
    switch(request_status.code){
        case SrmStatus::SRM_REQUEST_QUEUED:
        case SrmStatus::SRM_REQUEST_INPROGRESS:
        case SrmStatus::SRM_SUCCESS:
        case SrmStatus::SRM_FAILURE:
        case SrmStatus::SRM_PARTIAL_SUCCESS:
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_SPACE_LIFETIME_EXPIRED:
        case SrmStatus::SRM_EXCEED_ALLOCATION:
        case SrmStatus::SRM_NO_USER_SPACE:
        case SrmStatus::SRM_NO_FREE_SPACE:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_ABORTED:
        case SrmStatus::SRM_REQUEST_TIMED_OUT:
        case SrmStatus::SRM_NOT_SUPPORTED:
            // Ok
        break;
        default:
            // Invalid result returned by the SRM: set it to generic 
            // failure
            failure_for_invalid_result(request_status);
        break;
    }
}

/**
 * Check Postconditions - PrepareToPut
 */
template<>
inline void postconditions(PrepareToPut& request, SrmStatus& request_status,const ::srm2::srm2__srmPrepareToPutResponse * srm_result)  /*throw (LogicError, SrmBadResponse)*/{
    put_postconditions(request,request_status);
}
/**
 * Check Postconditions - StatusOfPutRequest
 */
template<>
inline void postconditions(PrepareToPut& request, SrmStatus& request_status,const ::srm2::srm2__srmStatusOfPutRequestResponse * srm_result)  /*throw (LogicError, SrmBadResponse)*/{
    put_postconditions(request,request_status);
}
/**
 * Check Postconditions - PutDone
 */
template<>
inline void postconditions(PrepareToPut& request, SrmStatus& request_status,const ::srm2::srm2__srmPutDoneResponse * srm_result)  /*throw (LogicError, SrmBadResponse)*/{
    // Post-condition check
    switch(request_status.code){
        case SrmStatus::SRM_SUCCESS:
        case SrmStatus::SRM_PARTIAL_SUCCESS:
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_REQUEST_TIMED_OUT:
        case SrmStatus::SRM_ABORTED:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_FAILURE:
        case SrmStatus::SRM_NOT_SUPPORTED:
            // Ok
        break;
        default:
            // Invalid result returned by the SRM: set it to generic 
            // failure
            failure_for_invalid_result(request_status);
        break;
    }
}
/**
 * Check Postconditions - AbortRequest
 */
template<>
inline void postconditions(PrepareToPut& request, SrmStatus& request_status,const ::srm2::srm2__srmAbortRequestResponse * srm_result)  /*throw (LogicError, SrmBadResponse)*/{
    // Post-condition check
    check_abort_postconditions(request_status);
}

/*
 * Get File State From PrepareToPut state - Used in PrepareToPut and StatusOfPutRequest
 */
inline void file_from_put_request_status(const PrepareToPut& request, const SrmStatus& request_status, PutFileRequest& file) {
    
    // Check the result and get the default file status
    switch(request_status.code){
        case SrmStatus::SRM_REQUEST_QUEUED:
        case SrmStatus::SRM_REQUEST_INPROGRESS:
        case SrmStatus::SRM_FAILURE:
        case SrmStatus::SRM_ABORTED:
            file.status.code = request_status.code;
        break;
        case SrmStatus::SRM_SUCCESS:
            // TODO: does it make sense? a TURL is required!
            file.status.code = SrmStatus::SRM_SPACE_AVAILABLE;
        break;
        case SrmStatus::SRM_PARTIAL_SUCCESS:
            // This should never happen
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_SPACE_LIFETIME_EXPIRED:
        case SrmStatus::SRM_EXCEED_ALLOCATION:
        case SrmStatus::SRM_NO_USER_SPACE:
        case SrmStatus::SRM_NO_FREE_SPACE:
        case SrmStatus::SRM_NOT_SUPPORTED:
        case SrmStatus::SRM_INTERNAL_ERROR:
            // Leave the file status undef
            file.status.code = SrmStatus::SRM_UNDEF;
        break;
        default:
            // This should never happen
        break;
    }
}

/**
 * Get File State From Request state - PrepareToPut
 */
template<>
inline void file_from_request_status(const PrepareToPut& request, const SrmStatus& request_status, PutFileRequest& file, const ::srm2::srm2__srmPrepareToPutResponse * srm_result) {
    file_from_put_request_status(request,request_status,file);
}
/**
 * Get File State From Request state - StatusOfPutRequest
 */
template<>
inline void file_from_request_status(const PrepareToPut& request, const SrmStatus& request_status, PutFileRequest& file, const ::srm2::srm2__srmStatusOfPutRequestResponse * srm_result) {
    file_from_put_request_status(request,request_status,file);
}
/**
 * Get File State From Request state - PutDone
 */
template<>
inline void file_from_request_status(const PrepareToPut& request, const SrmStatus& request_status, PutFileRequest& file, const ::srm2::srm2__srmPutDoneResponse * srm_result) {
    // Check the result and get the default file status
    switch(request_status.code){
        case SrmStatus::SRM_SUCCESS:
            file.status.code = SrmStatus::SRM_SUCCESS;
        break;
        case SrmStatus::SRM_FAILURE:
        case SrmStatus::SRM_ABORTED:
            file.status.code = request_status.code;
        break;
        case SrmStatus::SRM_PARTIAL_SUCCESS:
            // This should never happen
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_REQUEST_TIMED_OUT:
        case SrmStatus::SRM_NOT_SUPPORTED:
            // Leave the file status undef
            file.status.code = SrmStatus::SRM_UNDEF;
        break;
        default:
            // This should never happen
        break;
    }
}
/**
 * Get File State From Request state - AbortRequest
 */
template<>
inline void file_from_request_status(const PrepareToPut& request, const SrmStatus& request_status, PutFileRequest& file, const ::srm2::srm2__srmAbortRequestResponse * srm_result) {
    // Check the result and get the default file status
    switch(request_status.code){
        case SrmStatus::SRM_SUCCESS:
            file.status.code = SrmStatus::SRM_ABORTED;
        break;
        case SrmStatus::SRM_ABORTED:
            // This shoudl not be returned by the SRM, but it's still valid
        case SrmStatus::SRM_FAILURE:
            file.status.code = request_status.code;
        break;
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_NOT_SUPPORTED:
            // Leave the file status undef
            file.status.code = SrmStatus::SRM_UNDEF;
        break;
        default:
            // This should never happen
        break;
    }
}

/**
 * Return true if the File structure is related to the given file.
 * Used in PrepareToPut and StatusOfPutRequest
 */
template<>
inline bool match_file(const PutFileRequest& file, const ::srm2::srm2__TPutRequestFileStatus * file_stat) {
    bool result = false;
    if((file_stat != 0) && (file_stat->SURL == file.surl)){
        result = true;
    }
    return result;
}
/**
 * Return true if the File structure is related to the given file
 * Used in PutDone
 */
template<>
inline bool match_file(const PutFileRequest& file, const ::srm2::srm2__TSURLReturnStatus * file_stat) {
    bool result = false;
    if((file_stat != 0) && (file_stat->surl == file.surl)){
        result = true;
    }
    return result;
}

/**
 * Update Request Properties - PrepareToPut
 */    
template<>
inline void update_request_properties(PrepareToPut& request,const SrmStatus& status, const ::srm2::srm2__srmPrepareToPutResponse * srm_result) /*throw (LogicError, SrmBadResponse)*/{

    // Check Token
    if(0 != srm_result->requestToken){
        if(true == request.token().empty()){
            // Assign the token to the request
            request.token(*(srm_result->requestToken));
        }
    }
    // Update Additional Information
    if(0 != srm_result->remainingTotalRequestTime){
        request.remainingTotalRequestTime = *(srm_result->remainingTotalRequestTime);
    } else {
        request.remainingTotalRequestTime = -1;
    }
}
/**
 * Update Request Properties - StatusOfPutRequest
 */    
template<>
inline void update_request_properties(PrepareToPut& request,const SrmStatus& status, const ::srm2::srm2__srmStatusOfPutRequestResponse * srm_result) /*throw (LogicError, SrmBadResponse)*/{
    // Update Additional Information
    if(0 != srm_result->remainingTotalRequestTime){
        request.remainingTotalRequestTime = *(srm_result->remainingTotalRequestTime);
    } else {
        request.remainingTotalRequestTime = -1;
    }
}
/**
 * Update Request Properties - PutDone
 */    
template<>
inline void update_request_properties(PrepareToPut& request,const SrmStatus& status, const ::srm2::srm2__srmPutDoneResponse * srm_result) /*throw (LogicError, SrmBadResponse)*/{
    // Nothing To Do
}
/**
 * Update Request Properties - AbortRequest
 */    
template<>
inline void update_request_properties(PrepareToPut& request,const SrmStatus& status, const ::srm2::srm2__srmAbortRequestResponse * srm_result) /*throw (LogicError, SrmBadResponse)*/{
    // Nothing To Do
}
/*
 * Update Get File Properties. Used in PrepareToPut and StatusOfPutRequest
 */    
inline void update_put_file_properties(const PrepareToPut& request, const SrmStatus& request_status, PutFileRequest& file, const ::srm2::srm2__TPutRequestFileStatus * file_stat) /*throw (LogicError, SrmBadResponse)*/{
    if(0 == file_stat){
        // This should never happen: file_stat has been checked before
    } else {
        // Check if the file SURL is set otherwsie set it
        if(true == file.surl.empty()){
            file.surl = file_stat->SURL;
        }
        // Get File Status
        if(0 != file_stat->status){
            file.status.code = translate_StatusCode(file_stat->status->statusCode);
            if(0 != file_stat->status->explanation){
                file.status.message = *(file_stat->status->explanation);
            }
            // Check that the status code is valid
            switch(file.status.code){
                case SrmStatus::SRM_SPACE_AVAILABLE:
                case SrmStatus::SRM_REQUEST_QUEUED:
                case SrmStatus::SRM_REQUEST_INPROGRESS:
                case SrmStatus::SRM_SUCCESS:
                case SrmStatus::SRM_FILE_BUSY:
                case SrmStatus::SRM_FILE_IN_CACHE:
                case SrmStatus::SRM_INVALID_PATH:
                case SrmStatus::SRM_DUPLICATION_ERROR:
                case SrmStatus::SRM_AUTHORIZATION_FAILURE:
                case SrmStatus::SRM_ABORTED:
                case SrmStatus::SRM_RELEASED:
                case SrmStatus::SRM_FAILURE:
                // added on 25/04/07
                case SrmStatus::SRM_NO_FREE_SPACE:
                case SrmStatus::SRM_NO_USER_SPACE:
                case SrmStatus::SRM_REQUEST_SUSPENDED:
                    // Ok
                break;
                    // Note: Success and Request Suspended are not valid states
                    // for the PrepareToPut 
                default:
                    // Invalid result returned by the SRM
                    failure_for_invalid_result(file.status);
                break;
            }
        } else {
            // This should not happen: status is mandatory
            std::stringstream reason;
            reason << "The SRM returned no status for file [" 
                   << file_stat->SURL
                   << "] in PrepareToPut. According to the standard, the status is mandatory";
            throw SrmBadResponse(reason.str());
        }
        // File Size
        if(0 != file_stat->fileSize){
            file.size = *(file_stat->fileSize);
        }
        if(0 != file_stat->estimatedWaitTime){
            file.extimatedWaitTime = *(file_stat->estimatedWaitTime);
        }
        if(0 != file_stat->remainingPinLifetime){
            file.remainingPinTime = *(file_stat->remainingPinLifetime);
        }
        if(0 != file_stat->remainingFileLifetime){
            file.remainingFileTime = *(file_stat->remainingFileLifetime);
        }
        if(0 != file_stat->transferURL){
            file.turl = *(file_stat->transferURL);
        }
        // Ignore transferProtocolInfo
    }
}

/**
 * Update File Properties - PrepareToPut 
 */    
template<>
inline void update_file_properties(const PrepareToPut& request, const SrmStatus& request_status, PutFileRequest& file, const ::srm2::srm2__TPutRequestFileStatus * file_stat, const ::srm2::srm2__srmPrepareToPutResponse * srm_stat) /*throw (LogicError, SrmBadResponse)*/{
    update_put_file_properties(request,request_status,file,file_stat);
}
/**
 * Update File Properties - StatusOfPutRequest
 */    
template<>
inline void update_file_properties(const PrepareToPut& request, const SrmStatus& request_status, PutFileRequest& file, const ::srm2::srm2__TPutRequestFileStatus * file_stat, const ::srm2::srm2__srmStatusOfPutRequestResponse * srm_stat) /*throw (LogicError, SrmBadResponse)*/{
    update_put_file_properties(request,request_status,file,file_stat);
}
/**
 * Update File Properties - PutDone
 */    
template<>
inline void update_file_properties(const PrepareToPut& request, const SrmStatus& request_status, PutFileRequest& file, const ::srm2::srm2__TSURLReturnStatus * file_stat, const ::srm2::srm2__srmPutDoneResponse * srm_stat) /*throw (LogicError, SrmBadResponse)*/{
    if(0 == file_stat){
        // This should never happen: file_stat has been checked before
    } else {
        // Check if the file SURL is set otherwsie set it
        if(true == file.surl.empty()){
            file.surl = file_stat->surl;
        }
        // Get File Status
        if(0 != file_stat->status){
            file.status.code = translate_StatusCode(file_stat->status->statusCode);
            if(0 != file_stat->status->explanation){
                file.status.message = *(file_stat->status->explanation);
            }
            // Check that the status code  is valid
            switch(file.status.code){
                case SrmStatus::SRM_SUCCESS:
                case SrmStatus::SRM_INVALID_PATH:
                case SrmStatus::SRM_AUTHORIZATION_FAILURE:
                case SrmStatus::SRM_FILE_LIFETIME_EXPIRED:
                case SrmStatus::SRM_SPACE_LIFETIME_EXPIRED:
                case SrmStatus::SRM_ABORTED:
                case SrmStatus::SRM_FAILURE:
                    // Ok
                break;
                default:
                    // Invalid result returned by the SRM: set it to generic 
                    // failure
                    failure_for_invalid_result(file.status);
                break;
            }
        } else {
            // This should not happen: status is mandatory
            std::stringstream reason;
            reason << "The SRM returned no status for file [" 
                   << file_stat->surl
                   << "] in PutDone. According to the standard, the status is mandatory";
            throw SrmBadResponse(reason.str());
        }
    }
}

} // End namespace srm2_2
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_PUT_H_
