/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: srm common functions (rm specialization)
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: srm-common-rm.h,v 1.8 2008/03/13 16:32:45 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_RM_H_
#define GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_RM_H_

#include "SrmRm.h"
#include "srm-utility.h"
#include "srm-common-defs.h"

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm2_2{

/**
 * Check Preconditions
 */
template<>
inline void preconditions(const SrmRm& request, const ::srm2::srm2__srmRmResponse * srm_stat)  /*throw (LogicError, SrmBadResponse)*/{
    // Nothing To Check
}

/**
 * Check Postconditions
 */
template<>
inline void postconditions(SrmRm& request, SrmStatus& request_status, const ::srm2::srm2__srmRmResponse * srm_stat)  /*throw (LogicError, SrmBadResponse)*/{
    // Post-condition check
    switch(request_status.code){
        case SrmStatus::SRM_SUCCESS:
        case SrmStatus::SRM_FAILURE:
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_NOT_SUPPORTED:
        case SrmStatus::SRM_PARTIAL_SUCCESS:
            // OK
        break;
        default:
            // Invalid result returned by the SRM: set it to generic 
            // failure
            failure_for_invalid_result(request_status);
        break;
    }
}

/**
 * Get File State From Request state
 */
template<>
inline void file_from_request_status(const SrmRm& request, const SrmStatus& request_status, RmFileRequest& file, const ::srm2::srm2__srmRmResponse * srm_stat) {
    
    // Check the result and get the default file status
    switch(request_status.code){
        case SrmStatus::SRM_SUCCESS:
        case SrmStatus::SRM_FAILURE:
            file.status.code = request_status.code;
        break;
        case SrmStatus::SRM_AUTHENTICATION_FAILURE:
        case SrmStatus::SRM_AUTHORIZATION_FAILURE:
        case SrmStatus::SRM_INVALID_REQUEST:
        case SrmStatus::SRM_INTERNAL_ERROR:
        case SrmStatus::SRM_NOT_SUPPORTED:
        case SrmStatus::SRM_PARTIAL_SUCCESS:
            // Leave the file status undef
            file.status.code = SrmStatus::SRM_UNDEF;
        break;
        default:
            // This should never happen
        break;
    }
}

/**
 * Return true if the File structure is related to the given file
 */
template<>
inline bool match_file(const RmFileRequest& file, const ::srm2::srm2__TSURLReturnStatus * file_stat) {
    bool result = false;
    if((file_stat != 0) && (file_stat->surl == file.surl)){
        result = true;
    }
    return result;
}

/**
 * Update Request Properties
 */    
template<>
inline void update_request_properties(SrmRm& request,const SrmStatus& status, const ::srm2::srm2__srmRmResponse * srm_result) /*throw (LogicError, SrmBadResponse)*/{
    // Nothing to do
}

/**
 * Update File Proeprties
 */    
template<>
inline void update_file_properties(const SrmRm& request, const SrmStatus& request_status, RmFileRequest& file, const ::srm2::srm2__TSURLReturnStatus * file_stat, const ::srm2::srm2__srmRmResponse * srm_stat) /*throw (LogicError, SrmBadResponse)*/{
    if(0 == file_stat){
        // This should never happen: file_stat has been checked before
    } else {
        // Check if the file SURL is set otherwsie set it
        if(true == file.surl.empty()){
            file.surl = file_stat->surl;
        }
        // Set the file Status
        if(0 != file_stat->status){
            file.status.code = translate_StatusCode(file_stat->status->statusCode);
            if(0 != file_stat->status->explanation){
                file.status.message = *(file_stat->status->explanation);
            }
            // Check that the status code  is valid
            switch(file.status.code){
                case SrmStatus::SRM_SUCCESS:
                case SrmStatus::SRM_INVALID_PATH:
                case SrmStatus::SRM_AUTHORIZATION_FAILURE:
                case SrmStatus::SRM_FILE_LOST:
                case SrmStatus::SRM_FILE_UNAVAILABLE:
                case SrmStatus::SRM_FAILURE:
                    // Ok
                break;
                default:
                    // Invalid result returned by the SRM: set it to generic 
                    // failure
                    failure_for_invalid_result(file.status);
                break;
            }
        } else {
            // This should not happen: status is mandatory
            std::stringstream reason;
            reason << "The SRM returned no status for file [" 
                   << file_stat->surl
                   << "] in SrmRm. According to the standard, the status is mandatory";
            throw SrmBadResponse(reason.str());
        }
    }
}

} // End namespace srm2_2
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_COMMON_RM_H_
