/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: srm 2.2 utilities
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: srm-utility.h,v 1.6 2006/11/28 12:01:16 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_UTILITY_H_
#define GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_UTILITY_H_

#include "glite/data/srm/util/types.h"
#include "glite/data/srm2/cpp/envStub.h"
#include "glite/data/srm2/cpp/srm2Stub.h"

namespace glite {
namespace data  {
namespace srm   {
namespace util  {
namespace srm2_2{
   
/**
 * Set the status to generic failure 
 */
void failure_for_invalid_result(SrmStatus& status) /* throw ()*/;

/**
 * Check the validity of the request status after an abort
 */
void check_abort_postconditions(SrmStatus& status) /* throw ()*/;
        
// -----------------------------------------------------------------------------
// Srm 2.2 Enum translation routines
// -----------------------------------------------------------------------------

/**
 * Translate SrmStatus Code
 */
SrmStatus::Code translate_StatusCode(::srm2::srm2__TStatusCode code) /*throw (BadSrmResponse)*/;
    
/**
 * Translate FileStorageType
 */
::srm2::srm2__TFileStorageType translate_FileStorageType(FileStorageType::Value v) /* throw(LogicError) */;
FileStorageType::Value   translate_FileStorageType(::srm2::srm2__TFileStorageType v) /* throw(SrmBadResponse) */;

/**
 * Translate RetentionPolicy
 */
::srm2::srm2__TRetentionPolicy translate_RetentionPolicy(RetentionPolicy::Value v) /* throw(LogicError) */;
RetentionPolicy::Value   translate_RetentionPolicy(::srm2::srm2__TRetentionPolicy v) /* throw(SrmBadResponse) */;

/**
 * Translate AccessLatency
 */
::srm2::srm2__TAccessLatency translate_AccessLatency(AccessLatency::Value v) /* throw(LogicError) */;
AccessLatency::Value   translate_AccessLatency(::srm2::srm2__TAccessLatency v) /* throw(SrmBadResponse) */;

/**
 * Translate AccessPattern
 */
::srm2::srm2__TAccessPattern translate_AccessPattern(AccessPattern::Value v) /* throw(LogicError) */;

/**
 * Translate ConnectionType
 */
::srm2::srm2__TConnectionType translate_ConnectionType(ConnectionType::Value v) /* throw(LogicError) */;

/**
 * Translate OverwriteMode
 */
::srm2::srm2__TOverwriteMode translate_OverwriteMode(OverwriteMode::Value v) /* throw(LogicError) */;

/**
 * Translate FileLocality
 */
FileLocality::Value translate_FileLocality(::srm2::srm2__TFileLocality v) /* throw(SrmBadResponse) */;

/**
 * Translate FileType
 */
FileType::Value translate_FileType(::srm2::srm2__TFileType v) /* throw(SrmBadResponse) */;

/**
 * Translate PermissionMode
 */
PermissionMode::Value translate_PermissionMode(::srm2::srm2__TPermissionMode v) /* throw(SrmBadResponse) */;

} // End namespace srm2_2
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_2_2_SRM_VERSION_H_
