/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  gLite SRM Util - Get space metadata
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: getspacemetadata.cpp,v 1.2 2008/05/06 07:50:04 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "util-main.h"
#include "glite/data/srm/util/types.h"
#include "glite/data/srm/util/Context.h"
#include "glite/data/srm/util/GetSpaceMetaData.h"
#include "glite/data/srm/util/SrmExceptions.h"

using namespace glite::data::srm::util;

//
// Global variables
//

// Usage message
const char * const tool_usage = "TOKEN [TOKEN]";

// Help message
const char * const tool_help = "";


// get specific command line options
const char * const tool_options = "";

/**
 * print_request
 * print a request
 */
static void print_request(const GetSpaceMetaData& request, SrmStatus& result){
    info(" ");
    info("Status : %s", SrmStatus::toString(result.code));
    if(false == result.message.empty()){
        info("Message: %s", result.message.c_str());
    }
    
    // Display Tokens
    std::vector<MetaDataSpace>::const_iterator it;
    for(it = request.spaceDetails.begin(); it != request.spaceDetails.end(); ++it){
        
        const MetaDataSpace& data = (*it);
        
        info("\nSpace token : %s",data.spaceToken.c_str());
        info("\tStatus            : %s",SrmStatus::toString(data.status.code));
        if(false == (*it).status.message.empty()){
            info("\tMessage           : %s", data.status.message.c_str());
        }
        info("\tRetention policy  : %s",RetentionPolicy::toString(data.retentionPolicyInfo.retentionPolicy));
        info("\tAccess latency    : %s",AccessLatency::toString(data.retentionPolicyInfo.accessLatency));
        info("\tOwner             : %s",data.owner.c_str());
        info("\tTotal size        : %ld",data.totalSize);
        info("\tGuaranteed size   : %ld",data.guaranteedSize);
        info("\tUnused size       : %ld",data.unusedSize);
        info("\tLifetime assigned : %d",data.lifetimeAssigned);
        info("\tLifetime left     : %d",data.lifetimeLeft);
    }
}

/**
 * tool_doit
 * Run the request
 */
int tool_doit(Context::Ptr ctx, int argc, char *argv[]){
    
    if(argc <= 0){
        error("No Token(s) specified");
        print_usage();
        return EXIT_FAILURE;
    }
    
    // Create the GetSpaceMetaData Requestt Object
    boost::shared_ptr<GetSpaceMetaData> request(GetSpaceMetaData::create(ctx));
    
    // Set Request Parameters
    int num_tokens = argc;
    request->tokens.resize(num_tokens);
    for (int i=0;i < num_tokens;i++){
        request->tokens[i] = argv[i];
    }
    SrmStatus result;
    try{
        // Issue a GetSpaceMetaData Request
        result = request->execute();
    } catch(const SrmError& exc){
        error("GetSpaceMetaData Failed (SrmError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const RemoteError& exc){
        error("GetSpaceMetaData Failed (RemoteError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const SecurityError& exc){
        error("GetSpaceMetaData Failed (SecurityError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const SrmBadResponse& exc){
        error("GetSpaceMetaData Failed (SrmBadResponse): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const LogicError& exc){
        error("GetSpaceMetaData Failed (LogicError): %s",exc.what());
        return EXIT_FAILURE;
    }
    
    // Print request
    print_request(*request, result);
    
    return EXIT_SUCCESS;
}

/**
 * tool_parse_cmdline
 * Command line parsing
 */
int tool_parse_cmdline(int c, char *opt_arg) {
    // Nothing to do
    return -1;
}
