/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  gLite SRM Util - Performs a request to create a directory in the SRM
 *
 *  Authors: Paolo Badino <Paolo.Badino@cern.ch>
 *  Version info: $Id: getspacetokens.cpp,v 1.2 2008/05/06 07:50:04 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include "util-main.h"
#include "glite/data/srm/util/Context.h"
#include "glite/data/srm/util/GetSpaceTokens.h"
#include "glite/data/srm/util/SrmExceptions.h"

using namespace glite::data::srm::util;

//
// Global variables
//

// Usage message
const char * const tool_usage = "TOKEN_DESCRIPTION";

// Help message
const char * const tool_help = "";


// get specific command line options
const char * const tool_options = "";

/*
 * print_request
 * 
 * print a request
 */
static void print_request(const GetSpaceTokens& request, SrmStatus& result){
    
    // print the request token
    info(" ");
    info("Status : %s", SrmStatus::toString(result.code));
    if(false == result.message.empty()){
        info("Message: %s", result.message.c_str());
    }
    
    // Display Tokens
    info("\nDescription : %s",request.description.c_str());
    std::vector<std::string>::const_iterator it;
    for(it = request.tokens.begin(); it != request.tokens.end(); ++it){
        info("\t- %s",(*it).c_str());
    }
}

/*
 * tool_doit
 * 
 * Run the Get
 */
int tool_doit(Context::Ptr ctx, int argc, char *argv[]){
	
    if(argc <= 0){
        error("No Token Description specified");
        print_usage();
        return EXIT_FAILURE;
    }
    if(argc > 1){
        error("Too many Token Descriptions specified");
        print_usage();
        return EXIT_FAILURE;
    }
    
    // Create the GetSpaceTokens Requestt Object
    boost::shared_ptr<GetSpaceTokens> request(GetSpaceTokens::create(ctx));
    
    // Set Request Parameters
    request->description = argv[0];

    SrmStatus result;
    try{
        // Issue a GetSpaceTokens Request
        result = request->execute();
    } catch(const SrmError& exc){
        error("GetSpaceTokens Failed (SrmError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const RemoteError& exc){
        error("GetSpaceTokens Failed (RemoteError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const SecurityError& exc){
        error("GetSpaceTokens Failed (SecurityError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const SrmBadResponse& exc){
        error("GetSpaceTokens Failed (SrmBadResponse): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const LogicError& exc){
        error("GetSpaceTokens Failed (LogicError): %s",exc.what());
        return EXIT_FAILURE;
    }
    
    // Print request
    print_request(*request, result);
    
	return EXIT_SUCCESS;
}

/*
 * tool_parse_cmdline
 * 
 * Command line parsing
 */
int tool_parse_cmdline(int c, char *opt_arg) {
    // Nothing to do
	return -1;
}
