/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  gLite SRM Util - srmLs common functions
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: ls-common.h,v 1.2 2009/04/08 16:25:28 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_LS_COMMON_H_
#define GLITE_DATA_SRM_UTIL_CPP_LS_COMMON_H_

#include "glite/data/srm/util/SrmLs.h"
#include "ls_CommonOperations.h"

template<> struct Setup<SrmLs>
{
	Setup(int argc, char **argv)
		: _argc(argc), _argv(argv) {}

	void setup_request(SrmLs& request, const CommandLineParameters& pars)
	{
	    request.surls.clear();

	    for(int i = 0; i < _argc; ++i){
	        request.surls.push_back(_argv[i]);
	    }

	    request.fullDetailedList = pars.g_fullDetailedList;
	    request.allLevelRecursive = pars.g_allLevelRecursive;
	    request.numOfLevels = pars.g_numOfLevels;

	    if (pars.g_offset >= 0){
	        request.offset = pars.g_offset;
	    }
	    if (pars.g_count >= 0){
	        request.count = pars.g_count;
	    }
	}

private:
	int _argc;
	char **_argv;
};

struct SrmLsTraits : public SrmTraits<SrmLs>
{
	static char* Label() {return "SrmLs";}
	static void PrintRequest(const SrmLs& request);
};

#endif //GLITE_DATA_SRM_UTIL_CPP_LS_COMMON_H_
