/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  gLite SRM Util - Performs an srmLs request
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: ls.cpp,v 1.2 2009/04/08 16:25:28 molnarzs Exp $
 *  Release: $Name:  $
 *
 */

#include "util-main.h"
#include "ls-common.h"
#include <stdlib.h>

// Help message
const char * const tool_help =
    "\t-f\t\tFull detailed list\n"
    "\t-a\t\tAll levels recursive\n"
    "\t-n NUM\t\tnum of levels (default = 1)\n"
    "\t-o OFFSET\toffset (default = not specified)\n"
    "\t-c COUNT\tcount (default = not specified)\n"
    "\t-b\t\tRun the get in blocking mode\n";

// get specific command line options
const char * const tool_options = "fan:o:c:b";

// Usage message
const char * const tool_usage = "SURL [SURL]";

int tool_parse_cmdline(int c, char *opt_arg)
{
    switch(c)
    {
        case 'f':
        	g_parameters.g_fullDetailedList = true;
            return 0;
        case 'a':
        	g_parameters.g_allLevelRecursive = true;
            return 0;
        case 'n':
        	g_parameters.g_numOfLevels = atoi(opt_arg);
            return (g_parameters.g_numOfLevels >= 0) ? 0 : -1;
        case 'o':
        	g_parameters.g_offset = atoi(opt_arg);
            return (g_parameters.g_offset >= 0) ? 0 : -1;
        case 'c':
        	g_parameters.g_count = atoi(opt_arg);
            return (g_parameters.g_count >= 0) ? 0 : -1;
        case 'b':
        	g_parameters.g_blocking = true;
            return 0;
        default:
            break;
    }
    return -1;
}

struct SrmLsExecuteTraits
	: public SrmLsTraits,
	  public SrmExecuteTraits<SrmLs>
{
	SrmLsExecuteTraits(int argc, char *argv[])
		: SrmExecuteTraits<SrmLs>(argc, argv) {}
};

int tool_doit(Context::Ptr ctx, int argc, char *argv[])
{
	return tool_doit_general<SrmLsExecuteTraits>(ctx, argc, argv);
}
