/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  gLite SRM Util - Performs a request to ping the SRM
 *
 *  Authors: Paolo Badino <Paolo.Badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ping.cpp,v 1.3 2009/02/10 08:03:19 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include "util-main.h"
#include "glite/data/srm/util/Context.h"
#include "glite/data/srm/util/SrmPing.h"
#include "glite/data/srm/util/SrmExceptions.h"

using namespace glite::data::srm::util;

//
// Global variables
//

// Usage message
const char * const tool_usage = "";

// Help message
const char * const tool_help = "";


// get specific command line options
const char * const tool_options = "";

/*
 * print_request
 * 
 * print a request
 */
static void print_request(const SrmPing& request, SrmStatus& result){
    
    // print the request token
    info(" ");
    info("Status    : %s", SrmStatus::toString(result.code));
    if(false == result.message.empty()){
        info("Message   : %s", result.message.c_str());
    }
    info("Version   : %s", request.versionInfo.c_str());
    
    if(false == request.otherInfo.empty()){
        info("Extra Info:");
        ExtraInfo::const_iterator it;
        for(it = request.otherInfo.begin(); it != request.otherInfo.end(); ++it){
            info("\t%s: %s",(*it).first.c_str(),(*it).second.c_str());
        }
    }
}

/*
 * tool_doit
 * 
 * Run the Get
 */
int tool_doit(Context::Ptr ctx, int argc, char *argv[]){

    if(argc >= 1){
        error("Too many arguments specified");
        print_usage();
        return EXIT_FAILURE;     
    }
    // Create the Get Requestt Object
    boost::shared_ptr<SrmPing> request(SrmPing::create(ctx));
    
    SrmStatus result;
    try{
        // Issue a SrmRm Request
        result = request->execute();
    } catch(const SrmError& exc){
        error("SrmPing Failed (SrmError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const RemoteError& exc){
        error("SrmPing Failed (RemoteError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const SecurityError& exc){
        error("SrmPing Failed (SecurityError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const SrmBadResponse& exc){
        error("SrmPing Failed (SrmBadResponse): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const LogicError& exc){
        error("SrmPing Failed (LogicError): %s",exc.what());
        return EXIT_FAILURE;
    }
    
    // Print request
    print_request(*request, result);
    
	return EXIT_SUCCESS;
}

/*
 * tool_parse_cmdline
 * 
 * Command line parsing
 */
int tool_parse_cmdline(int c, char *opt_arg) {
    // Nothing to do
	return -1;
}
