/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  gLite SRM Util - status of a put request 
 *
 *  Authors: Paolo Badino <Paolo.Badino@cern.ch>
 *  Version info: $Id: put-status.cpp,v 1.1 2007/02/01 11:00:21 badino Exp $
 *  Release: $Name:  $
 *
 */

#include "util-main.h"
#include "put-common.h"

using namespace glite::data::srm::util;

//
// Global variables
//
static const unsigned int DEFAULT_SLEEP_TIME = 10;

// Usage message
const char * const tool_usage = "TOKEN";

// Help message
const char * const tool_help = "\n";


// get specific command line options
const char * const tool_options = "";

/*
 * tool_doit
 * 
 * Run the Get
 */
int tool_doit(Context::Ptr ctx, int argc, char *argv[]){
	
    if(argc <= 0){
        error("No Request Token specified");
        print_usage();
        return EXIT_FAILURE;
    }
    if(argc > 1){
        error("Too many Request Tokens specified");
        print_usage();
        return EXIT_FAILURE;
    }
    const char * token = argv[0];
    
    // Create the Get Requestt Object
    boost::shared_ptr<PrepareToPut> request(PrepareToPut::create(ctx, token));
    
    // Set Request Parameters
    try{
        // Update the request status
        request->update();
    } catch(const SrmError& exc){
        error("Get status of PrepareToPut Failed (SrmError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const RemoteError& exc){
        error("Get status of PrepareToPut Failed (RemoteError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const SecurityError& exc){
        error("Get status of PrepareToPut Failed (SecurityError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const SrmBadResponse& exc){
        error("Get status of PrepareToPut Failed (SrmBadResponse): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const LogicError& exc){
        error("Get status of PrepareToPut Failed (LogicError): %s",exc.what());
        return EXIT_FAILURE;
    }
    
    // Print request
    print_request(*request);
        
	return EXIT_SUCCESS;
}

/*
 * tool_parse_cmdline
 * 
 * Command line parsing
 */
int tool_parse_cmdline(int c, char *opt_arg) {
	switch(c)
	{
		default:
			break;
	}
	return -1;
}
