/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  gLite SRM Util - Performs a request to delete a file(s) from the SRM
 *
 *  Authors: Paolo Badino <Paolo.Badino@cern.ch>
 *  Version info: $Id: rm.cpp,v 1.2 2008/05/06 07:50:04 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include "util-main.h"
#include "glite/data/srm/util/Context.h"
#include "glite/data/srm/util/SrmRm.h"
#include "glite/data/srm/util/SrmExceptions.h"

using namespace glite::data::srm::util;

//
// Global variables
//
static const unsigned int DEFAULT_SLEEP_TIME = 10;

// Usage message
const char * const tool_usage = "SURL [SURL]";

// Help message
const char * const tool_help = "";


// get specific command line options
const char * const tool_options = "";

/*
 * print_request
 * 
 * print a request
 */
static void print_request(const SrmRm& request, SrmStatus& result){
    
    // print the request token
    info(" ");
    info("Status : %s", SrmStatus::toString(result.code));
    if(false == result.message.empty()){
        info("Message: %s", result.message.c_str());
    }
    
    // Display file Statuses
    std::vector<RmFileRequest>::const_iterator it;
    for(it = request.files.begin(); it != request.files.end(); ++it){
        info("\nFile : %s",(*it).surl.c_str());
        info("\tStatus           : %s",SrmStatus::toString((*it).status.code));
        if(false == (*it).status.message.empty()){
            info("\tMessage          : %s", (*it).status.message.c_str());
        }
    }
}

/*
 * tool_doit
 * 
 * Run the Get
 */
int tool_doit(Context::Ptr ctx, int argc, char *argv[]){
	
    if(argc <= 0){
        error("No Surl specified");
        print_usage();
        return EXIT_FAILURE;
    }
    
    // Create the Get Requestt Object
    boost::shared_ptr<SrmRm> request(SrmRm::create(ctx));
    
    // Set Request Parameters
    request->files.resize(argc);
    for(int i = 0; i < argc; ++i){
        request->files[i].surl = argv[i];
    }
    SrmStatus result;
    try{
        // Issue a SrmRm Request
        result = request->execute();
    } catch(const SrmError& exc){
        error("SrmRm Failed (SrmError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const RemoteError& exc){
        error("SrmRm Failed (RemoteError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const SecurityError& exc){
        error("SrmRm Failed (SecurityError): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const SrmBadResponse& exc){
        error("SrmRm Failed (SrmBadResponse): %s",exc.what());
        return EXIT_FAILURE;
    } catch(const LogicError& exc){
        error("SrmRm Failed (LogicError): %s",exc.what());
        return EXIT_FAILURE;
    }
    
    // Print request
    print_request(*request, result);
    
	return EXIT_SUCCESS;
}

/*
 * tool_parse_cmdline
 * 
 * Command line parsing
 */
int tool_parse_cmdline(int c, char *opt_arg) {
    // Nothing to do
	return -1;
}
