/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  gLite SRM Clients - Main function for the tools
 *
 *  Authors: Paolo Badino <Paolo.Badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: util-main.h,v 1.2 2008/05/06 07:50:04 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_SRM_UTIL_CPP_UTIL_MAIN_H_
#define GLITE_DATA_SRM_UTIL_CPP_UTIL_MAIN_H_

#include "glite/data/srm/util/Context.h"

/**********************************************************************
 * Constants
 */


/**********************************************************************
 * Data type definitions
 */


/**********************************************************************
 * Prototypes
 */

/* Parse tool-specific command line option */
int tool_parse_cmdline(int option, char *opt_arg);

/* Perform tool operation */
int tool_doit(glite::data::srm::util::Context::Ptr ctx, int argc, char *argv[]);

/**********************************************************************
 * Utility functions
 */

/* print program usage */
void print_usage();

/* Print a debug message */
void debug(const char *fmt, ...) __attribute__((format (printf, 1, 2)));

/* Print an informational message */
void info(const char *fmt, ...) __attribute__((format (printf, 1, 2)));

/* Print an error message */
void error(const char *fmt, ...) __attribute__((format (printf, 1, 2)));

/**********************************************************************
 * Global variables exported by the tool
 */

/* Tool-specific command line options */
extern const char * const tool_options;

/* Tool-specific help message */
extern const char * const tool_help;

/* Tool-specific usage instructions */
extern const char * const tool_usage;

/**********************************************************************
 * Global variables exported by the main module
 */

/* Verbosity level */
extern int verbose_flag;

/* Request quiet operation */
extern int quiet_flag;

#endif //GLITE_DATA_SRM_UTIL_CPP_UTIL_MAIN_H_
