/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SrmError Exception Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: types.cpp,v 1.3 2008/10/15 07:06:31 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/srm/util/types.h"

// Macro used for the enumeration
#define EVAL_CASE(name) case name: return #name;
#define EVAL_CASE_DEFAULT(name) case name: default: return #name;
    
using namespace glite::data::srm::util;

/**
 * Enum to string convertion function
 */
const char * AccessPattern::toString(AccessPattern::Value value) /*throw()*/{
    switch(value){
    EVAL_CASE(TRANSFER_MODE)
    EVAL_CASE(PROCESSING_MODE)
    EVAL_CASE_DEFAULT(UNDEF)
    }
}

/**
 * Enum to string convertion function
 */
const char * ConnectionType::toString(ConnectionType::Value value) /*throw()*/{
    switch(value){
    EVAL_CASE(WAN)
    EVAL_CASE(LAN)
    EVAL_CASE_DEFAULT(UNDEF)
    }
}

/**
 * Enum to string convertion function
 */
const char * FileStorageType::toString(FileStorageType::Value value) /*throw()*/{
    switch(value){
    EVAL_CASE(VOLATILE)
    EVAL_CASE(DURABLE)
    EVAL_CASE(PERMANENT)
    EVAL_CASE_DEFAULT(DEFAULT)
    }
}

/**
 * Enum to string convertion function
 */
const char * RetentionPolicy::toString(RetentionPolicy::Value value) /*throw()*/{
    switch(value){
    EVAL_CASE(REPLICA)
    EVAL_CASE(OUTPUT)
    EVAL_CASE(CUSTODIAL)
    EVAL_CASE_DEFAULT(DEFAULT)
    }
}

/**
 * Enum to string convertion function
 */
const char * AccessLatency::toString(AccessLatency::Value value) /*throw()*/{
    switch(value){
    EVAL_CASE(ONLINE)
    EVAL_CASE(NEARLINE)
    EVAL_CASE_DEFAULT(DEFAULT)
    }
}

/**
 * Enum to string convertion function
 */
const char * OverwriteMode::toString(OverwriteMode::Value value) /*throw()*/{
    switch(value){
    EVAL_CASE(ALWAYS)
    EVAL_CASE(WHEN_FILES_ARE_DIFFERENT)
    EVAL_CASE_DEFAULT(NEVER)
    }
}
/**
 * Enum to string convertion function
 */
const char * FileLocality::toString(FileLocality::Value value) /*throw()*/{
    switch(value){
    EVAL_CASE(ONLINE)
    EVAL_CASE(NEARLINE)
    EVAL_CASE(ONLINE_AND_NEARLINE)
    EVAL_CASE(LOST)
    EVAL_CASE(UNAVAILABLE)
    EVAL_CASE_DEFAULT(NONE)
    }
}

/**
 * Enum to string convertion function
 */
const char * FileType::toString(FileType::Value value) /*throw()*/{
    switch(value){
    EVAL_CASE(FILE)
    EVAL_CASE(DIRECTORY)
    EVAL_CASE(LINK)
    EVAL_CASE_DEFAULT(UNDEF)
    }
}

/**
 * Enum to string convertion function
 */
const char * PermissionMode::toString(PermissionMode::Value value) /*throw()*/{
    switch(value){
    EVAL_CASE(X)
    EVAL_CASE(W)
    EVAL_CASE(WX)
    EVAL_CASE(R)
    EVAL_CASE(RX)
    EVAL_CASE(RW)
    EVAL_CASE(RWX)
    EVAL_CASE_DEFAULT(NONE)
    }
}
    
/*
 * toString
 *
 * Retrun the string representation of a code
 */
const char * SrmStatus::toString(SrmStatus::Code code) /*throw()*/ {
    switch(code){
    EVAL_CASE(SRM_SUCCESS)
    EVAL_CASE(SRM_FAILURE)
    EVAL_CASE(SRM_AUTHENTICATION_FAILURE)
    EVAL_CASE(SRM_AUTHORIZATION_FAILURE)
    EVAL_CASE(SRM_INVALID_REQUEST)
    EVAL_CASE(SRM_INVALID_PATH)
    EVAL_CASE(SRM_FILE_LIFETIME_EXPIRED)
    EVAL_CASE(SRM_SPACE_LIFETIME_EXPIRED)
    EVAL_CASE(SRM_EXCEED_ALLOCATION)
    EVAL_CASE(SRM_NO_USER_SPACE)
    EVAL_CASE(SRM_NO_FREE_SPACE)
    EVAL_CASE(SRM_DUPLICATION_ERROR)
    EVAL_CASE(SRM_NON_EMPTY_DIRECTORY)
    EVAL_CASE(SRM_TOO_MANY_RESULTS)
    EVAL_CASE(SRM_INTERNAL_ERROR)
    EVAL_CASE(SRM_FATAL_INTERNAL_ERROR)
    EVAL_CASE(SRM_NOT_SUPPORTED)
    EVAL_CASE(SRM_REQUEST_QUEUED)
    EVAL_CASE(SRM_REQUEST_INPROGRESS)
    EVAL_CASE(SRM_REQUEST_SUSPENDED)
    EVAL_CASE(SRM_ABORTED)
    EVAL_CASE(SRM_RELEASED)
    EVAL_CASE(SRM_FILE_PINNED)
    EVAL_CASE(SRM_FILE_IN_CACHE)
    EVAL_CASE(SRM_SPACE_AVAILABLE)
    EVAL_CASE(SRM_LOWER_SPACE_GRANTED)
    EVAL_CASE(SRM_DONE)
    EVAL_CASE(SRM_PARTIAL_SUCCESS)
    EVAL_CASE(SRM_REQUEST_TIMED_OUT)
    EVAL_CASE(SRM_LAST_COPY)
    EVAL_CASE(SRM_FILE_BUSY)
    EVAL_CASE(SRM_FILE_LOST)
    EVAL_CASE(SRM_FILE_UNAVAILABLE)
    EVAL_CASE(SRM_CUSTOM_STATUS)  
    EVAL_CASE_DEFAULT(SRM_UNDEF)
    }
}

namespace std{

// overloads of std::ostream operator << 
std::ostream& operator << (std::ostream& stream,AccessPattern::Value value){
    return stream << AccessPattern::toString(value);
}

std::ostream& operator << (std::ostream& stream,ConnectionType::Value value){
    return stream << ConnectionType::toString(value);
}

std::ostream& operator << (std::ostream& stream,FileStorageType::Value value){
    return stream << FileStorageType::toString(value);
}

std::ostream& operator << (std::ostream& stream,RetentionPolicy::Value value){
    return stream << RetentionPolicy::toString(value);
}

std::ostream& operator << (std::ostream& stream,AccessLatency::Value value){
    return stream << AccessLatency::toString(value);
}

std::ostream& operator << (std::ostream& stream,OverwriteMode::Value value){
    return stream << OverwriteMode::toString(value);
}

std::ostream& operator << (std::ostream& stream,FileLocality::Value value){
    return stream << FileLocality::toString(value);
}

std::ostream& operator << (std::ostream& stream,FileType::Value value){
    return stream << FileType::toString(value);
}

std::ostream& operator << (std::ostream& stream,PermissionMode::Value value){
    return stream << PermissionMode::toString(value);
}

std::ostream& operator << (std::ostream& stream,SrmStatus::Code value){
    return stream << SrmStatus::toString(value);
}

std::ostream& operator << (std::ostream& stream,const SrmStatus& value){
    return stream << "[" << value.code << "] " << value.message;
}

} // namespace std;
