/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: GSoapContext 1.1 Test Case
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: GSoapContext11_TestCase.cpp,v 1.5 2009/04/08 16:28:19 molnarzs Exp $
 *  Release: $Name:  $
 *
 */

#include "GSoapContext11_TestCase.h"

using namespace glite::data::srm::util;
using namespace glite::data::srm::util::srm1_1;

/**
 * testGetError_InvalidRequest
 */
void GSoapContext11_TestCase::testGetError_InvalidRequest(){
    checkCodeForMessage("InvalidRequest_1","this is an invalid request",SrmStatus::SRM_INVALID_REQUEST);
    checkCodeForMessage("InvalidRequest_2","the request does not correspond to any known job",SrmStatus::SRM_INVALID_REQUEST);
    checkCodeForMessage("InvalidRequest_3","request [xxx-yyy] does not belong to",SrmStatus::SRM_INVALID_REQUEST);
    checkCodeForMessage("InvalidRequest_4","request [xxx-yyy] \n was not found",SrmStatus::SRM_INVALID_REQUEST);
    checkCodeForMessage("InvalidRequest_5","request [xxx-yyy] is not a pnfsfile",SrmStatus::SRM_INVALID_REQUEST);
}

/**
 * testGetError_AuthorizationFailure
 */
void GSoapContext11_TestCase::testGetError_AuthorizationFailure(){
    checkCodeForMessage("AuthorizationFailure_1","user has no permission to write in path",SrmStatus::SRM_AUTHORIZATION_FAILURE);
    checkCodeForMessage("AuthorizationFailure_2","path does not exist and user has no permissions to create it",SrmStatus::SRM_AUTHORIZATION_FAILURE);
    checkCodeForMessage("AuthorizationFailure_3","path is not subpath of the user`s root",SrmStatus::SRM_AUTHORIZATION_FAILURE);
    checkCodeForMessage("AuthorizationFailure_4","RequestFileStatus#-2139518116 failed with error:[  at Thu Mar 08 04:29:25 CST 2007 state RetryWait :  nonfatal error [org.dcache.srm.scheduler.NonFatalJobFailure: org.dcache.srm.SRMException: user`s path /pnfs/fnal.gov/usr/cms/WAX//resilient/dteam/RobustChallenge/CERN-April/volatile/1/maart/2007-03-04/file-1016-afb7c96b-d6bb-42ba-abd8-721c1bf9e36c.dat is not subpath of the user`s root] retrying",SrmStatus::SRM_AUTHORIZATION_FAILURE);
    checkCodeForMessage("AuthorizationFailure_5","the server sent an error response: 425 425 Cannot open port: java.lang.Exception: Pool manager error: No write pools configured for <dc_dteam:DTEAM-disk-only-sc@osm>",SrmStatus::SRM_AUTHORIZATION_FAILURE);
    checkCodeForMessage("AuthorizationFailure_6","RequestFileStatus#-2146941141 failed with error:[  at Tue Mar 27 21:52:04 CDT 2007 state Failed : user has no permission to write into path /pnfs/fnal.gov/usr/cms/WAX/resilient/dteam/RobustChallenge/CERN-April/volatile/1/maart/2007-03-15",SrmStatus::SRM_AUTHORIZATION_FAILURE);
    // this one must not match authorization failure patterns
    checkCodeForMessage("AuthorizationFailure_7","Remote method invocation failure on the remote SRM [httpg://ccsrm.in2p3.fr:8443/srm/managerv1]: [SRM_FAILURE] [SrmPing] failed: SOAP-ENV:Server.userException - java.rmi.RemoteException: SRM Authorization failed; nested exception is:\n        org.dcache.srm.SRMAuthorizationException: diskCacheV111.services.authorization.AuthorizationServiceException: authRequestID 1344697877 Message to gPlazma timed out for authentification of /C=CH/O=CERN/OU=GRID/CN=Maarten Litmaath 1689 and role null",SrmStatus::SRM_FAILURE);
}

/**
 * testGetError_DuplicationError
 */
void GSoapContext11_TestCase::testGetError_DuplicationError(){
    checkCodeForMessage("DuplicationError_1","RequestFileStatus#-2142565908 failed with error:[  at Wed Apr 25 17:00:49 CEST 2007 state Failed : GetStorageInfoFailed : file exists, cannot write",SrmStatus::SRM_DUPLICATION_ERROR);
}

/**
 * testGetError_LongMessage
 * check that a long message does not cause a "Memory exhausted" exception
 */
void GSoapContext11_TestCase::testGetError_LongMessage(){
    std::stringstream errstream;
    errstream << "RequestFileStatus#-2139518116 failed with error:[  at Thu Mar 08 04:29:25 CST 2007 state RetryWait :  nonfatal error [org.dcache.srm.scheduler.NonFatalJobFailure: org.dcache.srm.SRMException: user`s path /pnfs/fnal.gov/usr/cms/WAX//resilient/dteam/RobustChallenge/CERN-April/volatile/1/maart/2007-03-04/file-1016-afb7c96b-d6bb-42ba-abd8-721c1bf9e36c.dat is not subpath of the user`s root] retrying";
    for (int i = 0;i < 100;++i){
        errstream << " at Thu Mar 08 04:30:31 CST 2007 state RetryWait :  nonfatal error [org.dcache.srm.scheduler.NonFatalJobFailure: org.dcache.srm.SRMException: user`s path /pnfs/fnal.gov/usr/cms/WAX//resilient/dteam/RobustChallenge/CERN-April/volatile/1/maart/2007-03-04/file-1016-afb7c96b-d6bb-42ba-abd8-721c1bf9e36c.dat is not subpath of the user`s root] retrying"/* << std::endl*/;
    }
    errstream << " at Thu Mar 08 04:40:50 CST 2007 state Failed : number of retries exceeded: org.dcache.srm.scheduler.NonFatalJobFailure: org.dcache.srm.SRMException: user`s path /pnfs/fnal.gov/usr/cms/WAX//resilient/dteam/RobustChallenge/CERN-April/volatile/1/maart/2007-03-04/file-1016-afb7c96b-d6bb-42ba-abd8-721c1bf9e36c.dat is not subpath of the user`s root]";
    std::string errorMessage = errstream.str();
    getStatusForMessage(errorMessage);
}

/**
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GSoapContext11_TestCase, "GSoapContext11_TestCase" );

//------------------------------------------------------------------------------
// helper methods
//------------------------------------------------------------------------------

/**
 * getStatusForMessage
 * Get SrmStatus for error message
 * @param message [IN] the error message
 * @return corresponding SrmStatus
 */
SrmStatus GSoapContext11_TestCase::getStatusForMessage(const std::string& message){
    try{
        return GSoapContext::getError(message);
    } catch (const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch (...){
        CPPUNIT_FAIL("Unknown error");
    }
    return SrmStatus(); // avoid warning
}

/**
 * checkCodeForMessage
 * checks that the code of the SrmStatus for given message is the expected one
 * @param test_id [IN] an id for the test(used for assertion message)
 * @param message [IN] the error message
 * @param code [IN] the expected code
 */
void GSoapContext11_TestCase::checkCodeForMessage(
    const std::string& test_id,
    const std::string& message,
    const SrmStatus::Code& code){
    SrmStatus status = getStatusForMessage(message);
    if (code != status.code){
        std::stringstream str;
        str << test_id << " failed : got code " << SrmStatus::toString(status.code);
        str << " (expected " << SrmStatus::toString(code) << "; message length = "
            << message.length() <<")";
        CPPUNIT_FAIL(str.str().c_str());
    }
}
