/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: GSoapContext 1.1 Test Case
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: GSoapContext11_TestCase.h,v 1.1 2009/04/08 16:28:19 molnarzs Exp $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_SRM_UTIL_CPP_GSOAPCONTEXT11_TESTCASE_H_
#define GLITE_DATA_SRM_UTIL_CPP_GSOAPCONTEXT11_TESTCASE_H_

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <sstream>
#include <string>

#include "srm1.1/GSoapContext.h"

using namespace glite::data::srm::util;

class GSoapContext11_TestCase : public CppUnit::TestCase{

public:

    void setUp (){}
    void tearDown (){}

private:

    CPPUNIT_TEST_SUITE( GSoapContext11_TestCase );
    CPPUNIT_TEST( testGetError_InvalidRequest );
    CPPUNIT_TEST( testGetError_AuthorizationFailure );
    CPPUNIT_TEST( testGetError_DuplicationError );
    CPPUNIT_TEST( testGetError_LongMessage );
    CPPUNIT_TEST_SUITE_END();

    //--------------------------------------------------------------------------
    // test methods
    //--------------------------------------------------------------------------
    void testGetError_InvalidRequest ();
    void testGetError_AuthorizationFailure ();
    void testGetError_DuplicationError();
    void testGetError_LongMessage ();

    //--------------------------------------------------------------------------
    // helper methods
    //--------------------------------------------------------------------------

    /**
     * getStatusForMessage
     * Get SrmStatus for error message
     * @param message [IN] the error message
     * @return corresponding SrmStatus
     */
    SrmStatus getStatusForMessage(const std::string& message);

    /**
     * checkCodeForMessage
     * checks that the code of the SrmStatus for given message is the expected one
     * @param test_id [IN] an id for the test(used for assertion message)
     * @param message [IN] the error message
     * @param code [IN] the expected code
     */
    void checkCodeForMessage(
        const std::string& test_id,
        const std::string& message,
        const SrmStatus::Code& code);

};

#endif // #ifndef GLITE_DATA_SRM_UTIL_CPP_GSOAPCONTEXT11_TESTCASE_H_
