/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SRM 1.1 Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>,
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: Srm11_SrmLs.cpp,v 1.1 2009/04/08 16:28:20 molnarzs Exp $
 *  Release: $Name:  $
 *
 */

#include "glite/data/srm/util/SrmExceptions.h"
#include "glite/data/srm/util/SrmLs.h"

#include "Srm11_SrmLs.h"

using namespace glite::data::srm::util;

/*
 * The SRM 1.1 endpoint
 */
extern std::string g_srm_11_endpoint;

/*
 * setUp
 *
 * set-up test case
 */
void Srm11_SrmLs::setUp()
{
    CPPUNIT_ASSERT_MESSAGE("Invalid Endpoint", g_srm_11_endpoint.empty() == false);
    // Create Context
    m_ctx.reset(new test_helpers::TestContext());
    // Create an Srm Context
    m_ctx->endpoint = g_srm_11_endpoint;
    m_ctx->version  = "1.1";

   //TestBase::setUp();
}

/*
 * tearDown
 *
 * tear-down test case
 */
void Srm11_SrmLs::tearDown()
{
	//TestBase::tearDown();
	m_ctx.reset();
}

void Srm11_SrmLs::_check_supported(const std::string& version)
{
    // Create Context
    Context::Ptr ctx(new test_helpers::TestContext());
    ctx->endpoint = g_srm_11_endpoint;
    ctx->version  = version;
    TestBase::check_supported(ctx, v1_1);
    // Check supported methods
    CHECK_SUPPORTED(SrmLs, ctx, false);
}

/*
 * Test Supported Method
 */
void Srm11_SrmLs::testSupported()
{
    // Do it with different values of the version (test tag stripping)
    _check_supported("1.1.0");
    _check_supported("1.1");
    _check_supported("1.1.x");
}

/**
 * Test Ls Method
 */
void Srm11_SrmLs::testLs(){
    CHECK_SUPPORTED(SrmLs,m_ctx,false);
    // Create Request
    boost::shared_ptr<SrmLs> request(SrmLs::create(m_ctx));

    // Set Request Parameters
    request->surls.resize(1);
    request->surls[0] = "srm://localhost/srm/managerv1?SFN=/test/srm1/file001.txt";
    // Send Request
    request->execute();

    // Check Return Value
    CHECKSRMSTATUS("Invalid Result Code",request->status.code,SrmStatus::SRM_NOT_SUPPORTED);

    // Check file Properties
    CPPUNIT_ASSERT_MESSAGE("Invalid data returned", request->details.empty() == true);

}
