/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SRM 1.1 srmStat Test Cases
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>,
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: Srm11_SrmStat.cpp,v 1.1 2009/04/08 16:28:20 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/srm/util/Context.h"
#include "glite/data/srm/util/SrmExceptions.h"
#include "glite/data/srm/util/SrmStat.h"
#include "glite/data/agents/UserProxyEnv.h"

#include <boost/scoped_ptr.hpp>
#include <sstream>

#include "Srm11_SrmStat.h"

using namespace glite::data::srm::util;

using boost::scoped_ptr;

/*
 * The SRM 1.1 endpoint
 */
extern std::string g_srm_11_endpoint;

void Srm11_SrmStat::setUp()
{
    CPPUNIT_ASSERT_MESSAGE("Invalid Endpoint", g_srm_11_endpoint.empty() == false);
    m_ctx.reset(new test_helpers::TestContext());
    m_ctx->endpoint = g_srm_11_endpoint;
    m_ctx->version  = "1.1";

    TestSrmStat::setUp();
}

void Srm11_SrmStat::tearDown()
{
	TestSrmStat::tearDown();
}

void Srm11_SrmStat::_check_supported(const std::string& version)
{
    // Create Context
    Context::Ptr ctx(new test_helpers::TestContext());
    ctx->endpoint = g_srm_11_endpoint;
    ctx->version  = version;
    // Check supported methods
    TestBase::check_supported(ctx, v1_1);
    CHECK_SUPPORTED(SrmStat,ctx,true);
}

/*
 * Test Supported Method
 */
void Srm11_SrmStat::testSupported()
{
    // Do it with different values of the version (test tag stripping)
    _check_supported("1.1.0");
    _check_supported("1.1");
    _check_supported("1.1.x");
}
