/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SRM 1.1 Test Case
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: Srm11_TestCase.h,v 1.1 2009/04/08 16:28:20 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_SRM11_TESTCASE_H_
#define GLITE_DATA_SRM_UTIL_CPP_SRM11_TESTCASE_H_

#include "glite/data/srm/util/Context.h"
#include "TestBase_v1.h"

using namespace glite::data::srm::util;

class Srm11_TestCase : public TestBase_v1
{
public:
    virtual void setUp ();
    virtual void tearDown ();

private:
    CPPUNIT_TEST_SUITE( Srm11_TestCase);
    CPPUNIT_TEST( testSupported );
    CPPUNIT_TEST( testPing );
    CPPUNIT_TEST( testPing_SecurityError );
    CPPUNIT_TEST( testPing_RemoteError );
    CPPUNIT_TEST( testPut );
    CPPUNIT_TEST( testGet );
    CPPUNIT_TEST( testBringOnline );
    CPPUNIT_TEST( testCopy );
    CPPUNIT_TEST( testStat_AfterCopy );
    CPPUNIT_TEST( testRm);
    CPPUNIT_TEST( testRm_Again);
    CPPUNIT_TEST( testGetSpaceTokens);
    CPPUNIT_TEST( testGetSpaceMetaData );
    CPPUNIT_TEST( testMkDir);
    CPPUNIT_TEST( testRmDir_NoSuchDir);
    CPPUNIT_TEST( testRmDir_NonEmptyDir);
    CPPUNIT_TEST( testRm_FirstFile);
    CPPUNIT_TEST( testRmDir);
    CPPUNIT_TEST( testPut_forever_pending );
    CPPUNIT_TEST( testPut_duplication_error );
    CPPUNIT_TEST( testPut_no_free_space );
    CPPUNIT_TEST( testGet_forever_pending  );
    CPPUNIT_TEST( testCopy_forever_pending  );
    CPPUNIT_TEST( testBringOnline_forever_pending  );
    CPPUNIT_TEST( testPut_file_busy );
    CPPUNIT_TEST( testAbortFiles );
    CPPUNIT_TEST( testAbortFiles_AlreadyAborted );
    CPPUNIT_TEST( testAbortFiles_ErrorNoFile );
    CPPUNIT_TEST_SUITE_END();

    void testSupported();

    /**
     * Test Ping Method
     */
    void testPing();
    void testPing_SecurityError();
    void testPing_RemoteError();

    /**
     * Test Rm Method
     */
    void testRm();
    void testRm_Again();
    void testRm_FirstFile();

    /**
     * Test MkDir Method
     */
    void testMkDir();
    /**
     * Test RmDir Method
     */
    void testRmDir();
    void testRmDir_NoSuchDir();
    void testRmDir_NonEmptyDir();
    /**
     * Test GetSpaceTokens Method
     */
    void testGetSpaceTokens();
    /**
     * GetSpaceMetaData
     */
    void testGetSpaceMetaData();

    /**
     * Test AbortFiles Method
     */
    void testAbortFiles();
    void testAbortFiles_AlreadyAborted();
    void testAbortFiles_ErrorNoFile();

    // Test Asynch Request
    // -------------------------------------------------------------------------

    /**
     * Test Put Method
     */
    void testPut();
    void testPut_forever_pending();
    void testPut_no_free_space();
    void testPut_duplication_error();
    void testPut_file_busy();
    /**
     * Test Get Method
     */
    void testGet();
    void testGet_forever_pending();
    /**
     * Test BringOnline Method
     */
    void testBringOnline();
    void testBringOnline_forever_pending();
    /**
     * Test Copy Method
     */
    void testCopy();
    void testStat_AfterCopy();
    void testCopy_forever_pending ();

private:

    void putErrorUtil(
        const std::string& 		surl,
        const SrmStatus::Code& 	expected);

    void _check_supported(const std::string& version);
};

#endif //#ifndef GLITE_DATA_SRM_UTIL_CPP_SRM11_TESTCASE_H_
