/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SRM 2.2 srmStat Test Cases
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>,
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: Srm22_SrmStat.cpp,v 1.2 2009/06/21 17:09:31 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/srm/util/Context.h"
#include "glite/data/srm/util/SrmExceptions.h"
#include "glite/data/srm/util/SrmStat.h"
#include "glite/data/agents/UserProxyEnv.h"

#include <memory>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <boost/scoped_ptr.hpp>

#include "Srm22_SrmStat.h"

using namespace glite::data::srm::util;

using boost::scoped_ptr;

/*
 * The SRM 2.2 endpoint
 */
extern std::string g_srm_22_endpoint;

void Srm22_SrmStat::setUp()
{
    CPPUNIT_ASSERT_MESSAGE("Invalid Endpoint", g_srm_22_endpoint.empty() == false);
    TestBase::m_ctx.reset(new test_helpers::TestContext());
    m_ctx->endpoint = g_srm_22_endpoint;
    m_ctx->version  = "2.2";

    TestSrmStat::setUp();
    put_file(_testFilePath_asynch(), true);
}

void Srm22_SrmStat::tearDown()
{
	remove_file(_testFilePath_asynch());
	TestSrmStat::tearDown();
}

void Srm22_SrmStat::_check_supported(const std::string& version)
{
    // Create Context
    Context::Ptr ctx(new test_helpers::TestContext());
    ctx->endpoint = g_srm_22_endpoint;
    ctx->version  = version;
    TestBase::check_supported(ctx, v2_2);
    // Check supported methods
    CHECK_SUPPORTED(SrmStat, ctx, true);
}

void Srm22_SrmStat::testStat_asynch_request()
{
	std::string surl = _testFilePath_asynch();
    boost::shared_ptr<SrmStat> request(SrmStat::create(m_ctx));

    request->files.resize(1);
    request->files[0].surl = surl;

    SrmStatus result = request->execute();
    CHECKSRMSTATUS("Invalid Result Code", result.code, SrmStatus::SRM_SUCCESS);

    // Check file Properties
    CPPUNIT_ASSERT_MESSAGE("Invalid SURL",   request->files[0].surl == surl);
    CPPUNIT_ASSERT_MESSAGE("Invalid Status", (request->files[0].status.code == SrmStatus::SRM_SUCCESS) || (request->files[0].status.code == SrmStatus::SRM_UNDEF));
    CPPUNIT_ASSERT_MESSAGE("Null Size"   , request->files[0].size > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size", (size_t)request->files[0].size == strlen(SAMPLE_TEXT));
    CPPUNIT_ASSERT_MESSAGE("Invalid File Locality", request->files[0].fileLocality == FileLocality::ONLINE);
    CPPUNIT_ASSERT_MESSAGE("Invalid File Type", request->files[0].fileType == FileType::FILE);
    CPPUNIT_ASSERT_MESSAGE("Invalid Storage Type", request->files[0].fileStorageType == FileStorageType::VOLATILE);
}

const std::string& Srm22_SrmStat::_testFilePath_asynch()
{
	static const std::string s(testPath() + "this-one-returns-SRM_REQUEST_QUEUED-on-ls.dat");
	return s;
}
