/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SRM 2.2 Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>,
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: Srm22_TestCase.h,v 1.2 2009/06/21 17:09:31 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_SRM22_TESTCASE_H_
#define GLITE_DATA_SRM_UTIL_CPP_SRM22_TESTCASE_H_

#include "glite/data/srm/util/Context.h"
#include "TestBase_v2.h"

using namespace glite::data::srm::util;

class Srm22_TestCase : public TestBase_v2
{
public:
    /**
     * set-up test case
     */
    void setUp ();
    /**
     * teardown test case
     */
    void tearDown ();

private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( Srm22_TestCase);
        CPPUNIT_TEST( testSupported );
        CPPUNIT_TEST( testPing );
        CPPUNIT_TEST( testPing_SecurityError );
        CPPUNIT_TEST( testPing_RemoteError );
        CPPUNIT_TEST( testMkDir );
        CPPUNIT_TEST( testMkDir_Again );
        CPPUNIT_TEST( testMkDir_NoParentDir );
        CPPUNIT_TEST( testPut );
        CPPUNIT_TEST( testGet );
        CPPUNIT_TEST( testBringOnline );
        CPPUNIT_TEST( testCopy );
        CPPUNIT_TEST( testStat_AfterCopy );
        CPPUNIT_TEST( testRm );
        CPPUNIT_TEST( testRm_Again );
        CPPUNIT_TEST( testGetSpaceTokens_01 );
        CPPUNIT_TEST( testGetSpaceTokens_02 );
        CPPUNIT_TEST( testGetSpaceTokens_InvalidDescr );
        CPPUNIT_TEST( testGetSpaceMetaData );
        CPPUNIT_TEST( testGetSpaceMetaData_RequestFailure );
        CPPUNIT_TEST( testGetSpaceMetaData_InvalidRequestFailure );
        CPPUNIT_TEST( testGetSpaceMetaData_TokenFailure );
        CPPUNIT_TEST( testGetSpaceMetaData_InvalidTokenFailure );
        CPPUNIT_TEST( testRmDir_NoSuchDir );
        CPPUNIT_TEST( testRmDir_NonEmptyDir );
        CPPUNIT_TEST( testRm_FirstFile );
        CPPUNIT_TEST( testRmDir );
        CPPUNIT_TEST( testPut_forever_pending );
        CPPUNIT_TEST( testGet_forever_pending  );
        CPPUNIT_TEST( testCopy_forever_pending  );
        CPPUNIT_TEST( testBringOnline_forever_pending  );
        CPPUNIT_TEST( testPut_file_busy );
        CPPUNIT_TEST( testPut_no_free_space );
        CPPUNIT_TEST( testAbortFiles );
        CPPUNIT_TEST( testAbortFiles_AlreadyAborted );
        CPPUNIT_TEST( testAbortFiles_ErrorNoFile );
        CPPUNIT_TEST( testCopy_sourceSpaceToken );
        CPPUNIT_TEST( testCopy_copyPinLifeTime );
        CPPUNIT_TEST( testPut_copyPinLifeTime );
    CPPUNIT_TEST_SUITE_END();

    // Test Synch Request
    // -------------------------------------------------------------------------

    /**
     * Test Ping Method
     */
    void testPing();
    void testPing_SecurityError();
    void testPing_RemoteError();

    /**
     * Test Rm Method
     */
    void testRm();
    void testRm_Again();
    void testRm_FirstFile();

    /**
     * Test MkDir Method
     */
    void testMkDir();
    void testMkDir_Again();
    void testMkDir_NoParentDir();

    /**
     * Test RmDir Method
     */
    void testRmDir();
    void testRmDir_NoSuchDir();
    void testRmDir_NonEmptyDir();
    /**
     * Test GetSpaceTokens Method
     */
    void testGetSpaceTokens_01();
    void testGetSpaceTokens_02();
    void testGetSpaceTokens_InvalidDescr();
    /**
     * Test GetSpaceMetaData Method
     */
    void testGetSpaceMetaData();
    void testGetSpaceMetaData_RequestFailure();
    void testGetSpaceMetaData_InvalidRequestFailure();
    void testGetSpaceMetaData_TokenFailure();
    void testGetSpaceMetaData_InvalidTokenFailure();
    /**
     * Test AbortFiles Method
     */
    void testAbortFiles();
    void testAbortFiles_AlreadyAborted();
    void testAbortFiles_ErrorNoFile();

    // Test Asynch Request
    // -------------------------------------------------------------------------

    /**
     * Test Put Method
     */
    void testPut();
    void testPut_forever_pending();
    void testPut_file_busy();
    void testPut_no_free_space();
    void testPut_copyPinLifeTime();

    /**
     * Test Get Method
     */
    void testGet();
    void testGet_forever_pending();
    /**
     * Test BringOnline Method
     */
    void testBringOnline();
    void testBringOnline_forever_pending();
    /**
     * Test Copy Method
     */
    void testCopy();
    void testCopy_forever_pending();
    void testCopy_sourceSpaceToken();
    void testCopy_copyPinLifeTime();
    void testStat_AfterCopy();

    virtual void _check_supported(const std::string& version);
};

#endif //#ifndef GLITE_DATA_SRM_UTIL_CPP_SRM22_TESTCASE_H_
