/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: Common base of the SRM Util CPOP tests.
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: TestBase.h,v 1.3 2009/07/30 17:40:34 molnarzs Exp $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_SRM_UTIL_CPP_TESTBASE_H_
#define GLITE_DATA_SRM_UTIL_CPP_TESTBASE_H_

#include "glite/data/srm/util/Context.h"
#include "test_helpers.h"
#include "../src/ExponentialBackoffLogic.h"

using namespace glite::data::srm::util;

extern const char SAMPLE_TEXT[];

class TestBase : public CppUnit::TestCase
{
public:
	typedef enum {
		v1_1,
		v2_2
	} SrmVersion;

	/* Call it from the overriding setUp! */
	virtual void setUp();
	/* Call it from the overriding tearDown! */
	virtual void tearDown();

	/* Utility methods for the tests */
	void put_file(const std::string& surl, bool overWrite = false);
    void remove_file(const std::string& surl);
    void check_supported(Context::Ptr ctx, SrmVersion v);
    void make_dir(const std::string& surl, bool ignoreIfExists = false);
    void remove_dir(const std::string& surl, bool recursive = false, bool ignoreIfNotExists = false);
    const std::string& testPath();

    Context::Ptr m_ctx;

	struct AlgorithmConstants
	{
		static double c() { return 1;  /* sec */ }
		static double timeout() { return 14;  /* sec */}
		static double base() { return 2; }
		static double maxSleepTime() { return 30; /* sec */ }
	};

    boost::shared_ptr<glite::data::srm::util::details::IBackoffLogic> _logic;

protected:
    /* Must be defined by the children */
    virtual const std::string _endPointRoot() = 0;
    /* Relative to the test base path. Should be direct node of the base path! */
    virtual const std::string _testSpecificPath() = 0;

private:
    const std::string& _testBasePath();

};

#endif // #ifndef GLITE_DATA_SRM_UTIL_CPP_TESTBASE_H_
