/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: SRM 2.2 Test Case
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: TestSrmBusy.cpp,v 1.2 2009/07/30 17:40:34 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "TestSrmBusy.h"

#include "glite/data/srm/util/SrmLs.h"
#include "glite/data/srm/util/SrmStat.h"
#include "glite/data/srm/util/PrepareToGet.h"
#include "glite/data/srm/util/PrepareToPut.h"
#include "glite/data/srm/util/AbortFiles.h"
#include "glite/data/srm/util/BringOnline.h"
#include "glite/data/srm/util/GetSpaceMetaData.h"
#include "glite/data/srm/util/GetSpaceTokens.h"
#include "glite/data/srm/util/SrmCopy.h"
#include "glite/data/srm/util/SrmMkDir.h"
#include "glite/data/srm/util/SrmRm.h"
#include "glite/data/srm/util/SrmRmDir.h"

#include <boost/lexical_cast.hpp>

extern std::string g_srm_22_endpoint;

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<SrmLs>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->surls.push_back(fname);
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<SrmStat>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->files.resize(1);
    request->files[0].surl = fname;
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<SrmRm>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->files.resize(1);
    request->files[0].surl = fname;
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<PrepareToGet>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->files.resize(1);
    request->files[0].surl = fname;
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<AbortFiles>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->token = fname;
    request->files.resize(1);
    request->files[0].surl = fname;
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<PrepareToPut>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->files.resize(1);
    request->files[0].surl = fname;
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<BringOnline>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->files.resize(1);
    request->files[0].surl = fname;
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<GetSpaceMetaData>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->tokens.push_back(fname);
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<GetSpaceTokens>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->description = fname;
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<SrmCopy>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->files.resize(1);
    request->files[0].sourceSurl = fname;
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<SrmMkDir>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->directoryPath = fname;
}

template<>
void TestSrmBusy::_setupRequest(boost::shared_ptr<SrmRmDir>& request, const std::string& fname)
{
    request->setBackoffLogic(_logic);
    request->directoryPath = fname;
}

template<class REQUEST>
void TestSrmBusy::_executeSyncRequest(boost::shared_ptr<REQUEST>& request, SrmStatus::Code code)
{
	boost::posix_time::ptime start(boost::posix_time::second_clock::universal_time());
	SrmStatus result = request->execute();
	std::cout << "\n\tTest time: " << elapsedTime(start) << " seconds..." << std::endl;;
	CHECKSRMSTATUS("Invalid Result Code: ", result.code, code);
}

template<class REQUEST>
void TestSrmBusy::_executeAsyncRequest(boost::shared_ptr<REQUEST>& request, SrmStatus::Code code)
{
	boost::posix_time::ptime start(boost::posix_time::second_clock::universal_time());
	request->execute();
	std::cout << "\n\tTest time: " << elapsedTime(start) << " seconds..." << std::endl;;
	CHECKSRMSTATUS("Invalid Result Code: ", request->status.code, code);
}

void TestSrmBusy::setUp()
{
    CPPUNIT_ASSERT_MESSAGE("Invalid Endpoint", g_srm_22_endpoint.empty() == false);
    m_ctx.reset(new test_helpers::TestContext());
    m_ctx->endpoint = g_srm_22_endpoint;
    m_ctx->version  = "2.2";
    TestBase::setUp();
}

void TestSrmBusy::tearDown()
{
	TestBase::tearDown();
}

void TestSrmBusy::_check_supported(const std::string& version)
{
    // Create Context
    Context::Ptr ctx(new test_helpers::TestContext());
    ctx->endpoint = g_srm_22_endpoint;
    ctx->version  = version;
    TestBase::check_supported(ctx, v2_2);
    // Check supported methods
    CHECK_SUPPORTED(SrmLs, ctx, true);
    CHECK_SUPPORTED(SrmStat, ctx, true);
    CHECK_SUPPORTED(PrepareToGet, ctx, true);
    CHECK_SUPPORTED(PrepareToPut, ctx, true);
    CHECK_SUPPORTED(AbortFiles, ctx, true);
    CHECK_SUPPORTED(BringOnline, ctx, true);
    CHECK_SUPPORTED(GetSpaceMetaData, ctx, true);
    CHECK_SUPPORTED(GetSpaceTokens, ctx, true);
    CHECK_SUPPORTED(SrmCopy, ctx, true);
    CHECK_SUPPORTED(SrmMkDir, ctx, true);
    CHECK_SUPPORTED(SrmRm, ctx, true);
    CHECK_SUPPORTED(SrmRmDir, ctx, true);
}

template<class REQUEST>
void TestSrmBusy::_testCallReturns_IntErr_Base_Async()
{
	const std::string fname = "srm_busy:SRM_INTERNAL_ERROR:1:3:SRM_REQUEST_QUEUED:0:-1";
    boost::shared_ptr<REQUEST> request(REQUEST::create(m_ctx));
    this->_setupRequest(request, fname);
    this->_executeAsyncRequest(request);
}

template<class REQUEST>
void TestSrmBusy::_testCallReturns_IntErr_Base_Sync()
{
	const std::string fname = "srm_busy:SRM_INTERNAL_ERROR:1:3:SRM_REQUEST_QUEUED:0:-1";
    boost::shared_ptr<REQUEST> request(REQUEST::create(m_ctx));
    this->_setupRequest(request, fname);
    this->_executeSyncRequest(request);
}


void TestSrmBusy::testCallReturns_IntErr_AbortFiles()
{
	_testCallReturns_IntErr_Base_Sync<AbortFiles>();
}

void TestSrmBusy::testCallReturns_IntErr_Rm()
{
	_testCallReturns_IntErr_Base_Sync<SrmRm>();
}

void TestSrmBusy::testCallReturns_IntErr_Ls()
{
	_testCallReturns_IntErr_Base_Async<SrmLs>();
}

void TestSrmBusy::testCallReturns_IntErr_PrepareToGet()
{
	_testCallReturns_IntErr_Base_Async<PrepareToGet>();
}

void TestSrmBusy::testCallReturns_IntErr_Stat()
{
	_testCallReturns_IntErr_Base_Sync<SrmStat>();
}

void TestSrmBusy::testCallReturns_IntErr_PrepareToPut()
{
	_testCallReturns_IntErr_Base_Async<PrepareToPut>();
}

void TestSrmBusy::testCallReturns_IntErr_BringOnline()
{
	_testCallReturns_IntErr_Base_Async<BringOnline>();
}

void TestSrmBusy::testCallReturns_IntErr_GetSpaceMetaData()
{
	_testCallReturns_IntErr_Base_Sync<GetSpaceMetaData>();
}

void TestSrmBusy::testCallReturns_IntErr_GetSpaceTokens()
{
	_testCallReturns_IntErr_Base_Sync<GetSpaceTokens>();
}

void TestSrmBusy::testCallReturns_IntErr_Copy()
{
	_testCallReturns_IntErr_Base_Async<SrmCopy>();
}

void TestSrmBusy::testCallReturns_IntErr_MkDir()
{
	_testCallReturns_IntErr_Base_Sync<SrmMkDir>();
}

void TestSrmBusy::testCallReturns_IntErr_RmDir()
{
	_testCallReturns_IntErr_Base_Sync<SrmRmDir>();
}

void TestSrmBusy::testAsyncCallReturns_IntErr_ReqQueued_noEWT_Stat()
{
	const std::string fname = "srm_busy:SRM_INTERNAL_ERROR:1:1:SRM_REQUEST_QUEUED:2:-1";
    boost::shared_ptr<SrmStat> request(SrmStat::create(m_ctx));
    this->_setupRequest(request, fname);
    this->_executeSyncRequest(request);
}

void TestSrmBusy::testCallReturns_IntErr_GeneralTimeout_Rm()
{
	const std::string fname = "srm_busy:SRM_INTERNAL_ERROR:1:10:SRM_REQUEST_QUEUED:0:-1";
	boost::shared_ptr<SrmRm> request(SrmRm::create(m_ctx));
	this->_setupRequest(request, fname);
    this->_executeSyncRequest(request, SrmStatus::SRM_REQUEST_TIMED_OUT);
    _resetTestInServer();
}

void TestSrmBusy::testCallReturns_IntErr_GeneralTimeout_PrepareToGet()
{
	const std::string fname = "srm_busy:SRM_INTERNAL_ERROR:1:10:SRM_REQUEST_QUEUED:0:-1";
	boost::shared_ptr<PrepareToGet> request(PrepareToGet::create(m_ctx));
	this->_setupRequest(request, fname);
    this->_executeAsyncRequest(request, SrmStatus::SRM_REQUEST_TIMED_OUT);
    _resetTestInServer();
}

void TestSrmBusy::testCallReturns_IntErr_GeneralTimeout_Stat()
{
	const std::string fname = "srm_busy:SRM_INTERNAL_ERROR:1:10:SRM_REQUEST_QUEUED:0:-1";
	boost::shared_ptr<SrmStat> request(SrmStat::create(m_ctx));
	this->_setupRequest(request, fname);
    this->_executeSyncRequest(request, SrmStatus::SRM_REQUEST_TIMED_OUT);
    _resetTestInServer();
}

void TestSrmBusy::testCallReturns_IntErr_ReqQueued_noEWT_GeneralTimeout_Stat()
{
	const std::string fname = "srm_busy:SRM_INTERNAL_ERROR:1:2:SRM_REQUEST_QUEUED:10:-1";
	boost::shared_ptr<SrmStat> request(SrmStat::create(m_ctx));
	this->_setupRequest(request, fname);
    this->_executeSyncRequest(request, SrmStatus::SRM_REQUEST_TIMED_OUT);
    _resetTestInServer();
}

void TestSrmBusy::testAsynchCallReturns_ReqQueued_withEWT_PrepareToGet()
{
	const std::string fname = "srm_busy:SRM_INTERNAL_ERROR:1:0:SRM_REQUEST_QUEUED:3:3";
	boost::shared_ptr<PrepareToGet> request(PrepareToGet::create(m_ctx));
	this->_setupRequest(request, fname);
	boost::posix_time::ptime start(boost::posix_time::second_clock::universal_time());
	request->execute();
	int counter = 0;

	while (!request->backoff()) {
		++counter;
		CPPUNIT_ASSERT(counter < 4);
	}

	std::cout << "\n\tTest time: " << elapsedTime(start) << " seconds..." << std::endl;;
	CHECKSRMSTATUS("Invalid Result Code: ", request->status.code, SrmStatus::SRM_SUCCESS);
}

void TestSrmBusy::testAsynchCallReturns_ReqQueued_withEWT_GeneralTimeout_Copy()
{
	const std::string fname = "srm_busy:SRM_INTERNAL_ERROR:1:0:SRM_REQUEST_QUEUED:5:4";
	boost::shared_ptr<SrmCopy> request(SrmCopy::create(m_ctx));
	this->_setupRequest(request, fname);
	boost::posix_time::ptime start(boost::posix_time::second_clock::universal_time());
	request->execute();
	int counter = 0;

	while (!request->backoff()) {
		++counter;
		CPPUNIT_ASSERT(counter < 4);
	}

	std::cout << "\n\tTest time: " << elapsedTime(start) << " seconds..." << std::endl;;
	CHECKSRMSTATUS("Invalid Result Code: ", request->status.code, SrmStatus::SRM_REQUEST_TIMED_OUT);
}









void TestSrmBusy::testAsynchCallReturns_ReqQueued_PollReset()
{
	const std::string fname = "srm_busy:SRM_REQUEST_QUEUED:5:2:SRM_REQUEST_QUEUED:2:4";
	std::string surl = testPath() + fname;
	// The sync call
//	_testCallReturns_IntErr_sync<SrmStat>(surl, __LINE__);
}


void TestSrmBusy::_resetTestInServer()
{
	const std::string fname = "srm_busy:TEST_RESET";
	boost::shared_ptr<SrmStat> request(SrmStat::create(m_ctx));
	this->_setupRequest(request, fname);
	SrmStatus result = request->execute();
	CHECKSRMSTATUS("Invalid Result Code after test server reset: ", result.code, SrmStatus::SRM_SUCCESS);
}
