/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: Common base of the specific test handlers
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: TestHandler.h,v 1.2 2009/06/29 11:51:45 szamsu Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_TESTHANDLER_H_
#define GLITE_DATA_SRM_UTIL_CPP_TESTHANDLER_H_

#include <glite/data/srm2/cpp/envH.h>
#include <glite/data/srm2/cpp/srm2H.h>
#include <assert.h>

namespace srm2 {

template <class TestTraits> struct TestData {};

template <class SrmCallTraits, class TestTraits> struct TestDataExtractor {};

template <class SrmCallTraits, class TestTraits> struct TestHandlerFactory {};

template <class REQUEST, class REPLY>
struct SrmOpTraits
{
	typedef REQUEST RequestType;
	typedef REPLY ResponseType;

	SrmOpTraits(REQUEST& req, REPLY& rep, struct soap* s)
		: request(req),
		  reply(rep),
		  soap(s)
	{
		assert(s);
	}

	REQUEST& request;
	REPLY& reply;
	struct soap* soap;
};

/**
 * Interface for the common test case handlers.
 */
struct ITestHandler
{
	/// Handle a specific test case.
	virtual void handle() = 0;

	virtual ~ITestHandler() { /* EMPTY */ }
};

} // namespace srm2

#endif // #ifndef GLITE_DATA_SRM_UTIL_CPP_TESTHANDLER_H_
