/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Helpers for srm-util-cpp tests
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: test_helpers.h,v 1.6 2009/06/21 17:09:31 molnarzs Exp $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_SRM_UTIL_CPP_TEST_HELPERS_H_
#define GLITE_DATA_SRM_UTIL_CPP_TEST_HELPERS_H_

#include <glite/data/srm/util/Context.h>
#include <glite/data/srm/util/types.h>
#include <sstream>

#include "cppunit_wrapper.h"
#include <boost/date_time/posix_time/posix_time.hpp>

// helper macros for tests (do not use functions as otherwise assertion failures
// would report the name and line of the function instead of the test method)

#define FAIL_TEST(msg) {std::stringstream err; err << msg; CPPUNIT_FAIL(err.str().c_str());}

namespace u = glite::data::srm::util;

namespace test_helpers{
class TestContext : public u::Context{
protected:

    /**
     * before_call
     */
    virtual void before_call(
        const char * method);

    /**
     * on_success
     */
    virtual void on_success(
        const char * method,
        const char * address,
        const u::CallInfo * info);

    /**
     * on_failure
     */
    virtual void on_failure(
        const char * method,
        const char * address,
        const u::CallInfo * info);

    /**
     * on_warning
     */
    virtual void on_warning(
        const char * scope,
        const char * description,
        const char * address);

};

}

namespace{

const char * to_str(bool value){
    return value ? "true" : "false";
}

#define CHECKSRMSTATUS(msg,code,expected)\
    if (code != expected){\
        FAIL_TEST(msg << " (got " << glite::data::srm::util::SrmStatus::toString(code) << " expected " << glite::data::srm::util::SrmStatus::toString(expected) << ")");\
    }

#define CHECK_VALUE(value,expected)\
    if (value != expected){\
        FAIL_TEST("Invalid value for " << #value << ": got '" << value << "' expected '" << expected << "'");\
    }

#define CHECK_SUPPORTED(method,ctx,expected){\
    bool supported = method::isSupported(ctx);\
    if (supported != expected){\
        FAIL_TEST("Method " << #method << ", version " << ctx->version << ": isSupported returned " << to_str(supported) << " (expected " << to_str(expected) << ")");\
    }\
}

using namespace glite::data::srm::util;

} // end global namespace

void check_status(const char * msg,const SrmStatus::Code& value,const SrmStatus::Code& expected);

long elapsedTime(const boost::posix_time::ptime& startTime);

#endif //#ifndef GLITE_DATA_SRM_UTIL_CPP_TEST_HELPERS_H_
